@@ -3,7 +3,7 @@
 Spyder License Agreement (MIT License)
 --------------------------------------
 
-Copyright (c) 2009-2010 Pierre Raybaut
+Copyright (c) 2009-2011 Pierre Raybaut
 
 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
@@ -27,7 +27,22 @@ FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 """
 
-__version__ = '2.0.12'
+__version__ = '2.1.4'
 __license__ = __doc__
+__project_url__ = 'http://code.google.com/p/spyderlib'
+__forum_url__   = 'http://groups.google.com/group/spyderlib'
 
-#FIXME: Internal console MT: for i in range(100000): print i -> bug
+# Dear (Debian, RPM, ...) package makers, please feel free to customize the
+# following path to module's data (images) and translations:
+DATAPATH = LOCALEPATH = DOCPATH = ''
+
+def add_to_distribution(dist):
+    """Add package to py2exe/cx_Freeze distribution object
+    Extension to guidata.disthelpers"""
+    dist.add_modules('PyQt4')
+    for _modname in ('spyderlib', 'spyderplugins'):
+        dist.add_module_data_files(_modname, ("", ),
+                                   ('.png', '.svg', '.html', '.png', '.txt',
+                                    '.js', '.inv', '.ico', '.css', '.doctree',
+                                    '.qm', '.py',),
+                                   copy_to_root=False)
