@@ -1,11 +1,12 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2009-2010 Pierre Raybaut
+# Copyright © 2009-2011 Pierre Raybaut
 # Licensed under the terms of the MIT License
 # (see spyderlib/__init__.py for details)
 
 """
-Spyder configuration management
+Spyder GUI-related configuration management
+(for non-GUI configuration, see spyderlib/baseconfig.py)
 
 Important note regarding shortcuts:
     For compatibility with QWERTZ keyboards, one must avoid using the following
@@ -13,66 +14,97 @@ Important note regarding shortcuts:
         Ctrl + Alt + Q, W, F, G, Y, X, C, V, B, N
 """
 
-import os, sys, os.path as osp
-from datetime import date
-from PyQt4.QtGui import QLabel, QIcon, QPixmap, QFont, QFontDatabase
+import os
+import sys
+import os.path as osp
+
+from spyderlib.qt.QtGui import QLabel, QIcon, QPixmap, QFont, QFontDatabase
 
 # Local import
-from userconfig import UserConfig, get_home_dir, NoDefault
+from spyderlib.userconfig import UserConfig, get_home_dir, NoDefault
+from spyderlib.baseconfig import (SUBFOLDER, CHECK_ALL, EXCLUDED_NAMES,
+                                  get_module_data_path, _)
+from spyderlib.utils import iofuncs, codeanalysis
 
-DATA_DEV_PATH = osp.dirname(__file__)
-if not osp.isdir(DATA_DEV_PATH):
-    # Assuming py2exe distribution
-    DATA_DEV_PATH = osp.join(sys.prefix, "spyderlib")
-DOC_DEV_PATH = osp.join(DATA_DEV_PATH, 'doc')
-
-# The two following lines are patched when making the debian package:
-DATA_PATH = DATA_DEV_PATH # @@@DATA_PATH@@@
-DOC_PATH = DOC_DEV_PATH # @@@DOC_PATH@@@
-
-EDITABLE_TYPES = [int, long, float, list, dict, tuple, str, unicode, date]
-try:
-    from numpy import ndarray, matrix
-    EDITABLE_TYPES += [ndarray, matrix]
-except ImportError:
-    pass
-PICKLABLE_TYPES = EDITABLE_TYPES[:]
-try:
-    from PIL.Image import Image
-    EDITABLE_TYPES.append(Image)
-except ImportError:
-    pass
-
-# Max number of filter iterations for worskpace display:
-# (for workspace saving, itermax == -1, see Workspace.save)
-ITERMAX = -1 #XXX: To be adjusted if it takes too much to compute... 2, 3?
-
-EXCLUDED = ['nan', 'inf', 'infty', 'little_endian', 'colorbar_doc', 'typecodes',
-            '__builtins__', '__main__', '__doc__']
-
-def type2str(types):
-    """Convert types to strings"""
-    return [typ.__name__ for typ in types]
-
-def str2type(strings):
-    """Convert strings to types"""
-    return tuple( [eval(string) for string in strings] )
 
 SANS_SERIF = ['Sans Serif', 'DejaVu Sans', 'Bitstream Vera Sans',
-              'Bitstream Charter', 'Lucida Grande', 'Verdana', 'Geneva',
-              'Lucid', 'Arial', 'Helvetica', 'Avant Garde', 'sans-serif']
-SANS_SERIF.insert(0, unicode(QFont().family()))
+              'Bitstream Charter', 'Times', 'Lucida Grande', 'Calibri',
+              'MS Shell Dlg 2', 'Verdana', 'Geneva', 'Lucid', 'Arial',
+              'Helvetica', 'Avant Garde', 'sans-serif']
 
-MONOSPACE = ['Monospace', 'DejaVu Sans Mono', 'Consolas', 'Courier New',
+MONOSPACE = ['Monospace', 'DejaVu Sans Mono', 'Consolas', 'Monaco',
              'Bitstream Vera Sans Mono', 'Andale Mono', 'Liberation Mono',
-             'Monaco', 'Courier', 'monospace', 'Fixed', 'Terminal']
-MEDIUM = 10
-SMALL = 9
+             'Courier New', 'Courier', 'monospace', 'Fixed', 'Terminal']
+
+if sys.platform == 'darwin':
+    BIG = MEDIUM = SMALL = 12
+elif os.name == 'nt':
+    BIG = 12    
+    MEDIUM = 10
+    SMALL = 9
+else:
+    BIG = 12    
+    MEDIUM = 9
+    SMALL = 9
+
+# Extensions supported by Spyder's Editor
+EDIT_FILETYPES = (
+    (_("Python files"), ('.py', '.pyw', '.ipy')),
+    (_("Cython/Pyrex files"), ('.pyx', '.pxd', '.pxi')),
+    (_("C files"), ('.c', '.h')),
+    (_("C++ files"), ('.cc', '.cpp', '.cxx', '.h', '.hh', '.hpp', '.hxx')),
+    (_("OpenCL files"), ('.cl', )),
+    (_("Fortran files"), ('.f', '.for', '.f90', '.f95', '.f2k')),
+    (_("Patch and diff files"), ('.patch', '.diff', '.rej')),
+    (_("Batch files"), ('.bat', '.cmd')),
+    (_("Text files"), ('.txt',)),
+    (_("reStructured Text files"), ('.txt', '.rst')),
+    (_("gettext files"), ('.po', '.pot')),
+    (_("Web page files"), ('.css', '.htm', '.html',)),
+    (_("Configuration files"), ('.properties', '.session', '.ini', '.inf',
+                                '.reg', '.cfg', '.desktop')),
+                 )
+
+def _get_filters(filetypes):
+    filters = []
+    for title, ftypes in filetypes:
+        filters.append("%s (*%s)" % (title, " *".join(ftypes)))
+    filters.append("%s (*)" % _("All files"))
+    return "\n".join(filters)
+
+def _get_extensions(filetypes):
+    ftype_list = []
+    for _title, ftypes in filetypes:
+        ftype_list += list(ftypes)
+    return ftype_list
+
+EDIT_FILTERS = _get_filters(EDIT_FILETYPES)
+EDIT_EXT = _get_extensions(EDIT_FILETYPES)+['']
+
+# Extensions supported by Spyder's Variable explorer
+IMPORT_EXT = iofuncs.iofunctions.load_extensions.values()
+
+# Extensions that should be visible in Spyder's file/project explorers
+SHOW_EXT = ['.png', '.ico', '.svg']
+
+# Extensions supported by Spyder (Editor or Variable explorer)
+VALID_EXT = EDIT_EXT+IMPORT_EXT
+
+# Find in files include/exclude patterns
+INCLUDE_PATTERNS = [r'|'.join(['\\'+_ext+r'$' for _ext in EDIT_EXT if _ext])+\
+                    r'|README|INSTALL',
+                    r'\.pyw?$|\.ipy$|\.txt$|\.rst$',
+                    '.']
+EXCLUDE_PATTERNS = [r'\.pyc$|\.pyo$|\.orig$|\.hg|\.svn|build',
+                    r'\.pyc$|\.pyo$|\.orig$|\.hg|\.svn']
+
+# Name filters for file/project explorers (excluding files without extension)
+NAME_FILTERS = ['*' + _ext for _ext in VALID_EXT + SHOW_EXT if _ext]+\
+               ['README', 'INSTALL', 'LICENSE']
 
 DEFAULTS = [
             ('main',
              {
-              'translation': True,
               'vertical_dockwidget_titlebars': False,
               'vertical_tabs': False,
               'animated_docks': True,
@@ -85,6 +117,10 @@ DEFAULTS = [
               'lightwindow/is_maximized': False,
               'lightwindow/is_fullscreen': False,
               }),
+            ('quick_layouts',
+             {
+              'place_holder': '',
+              }),
             ('editor_appearance',
              {
               'cursor/width': 2,
@@ -151,33 +187,34 @@ DEFAULTS = [
               'show_elapsed_time': True,
               'show_icontext': False,
               'monitor/enabled': True,
-              'mpl_patch/enabled': True,
-              'mpl_patch/backend': 'Qt4Agg',
+              'pyqt/api_version': 0,
+              'pyqt/replace_inputhook': os.name == 'nt',
+              'pyqt/ignore_sip_setapi_errors': False,
+              'matplotlib/patch': True,
+              'matplotlib/backend/enabled': True,
+              'matplotlib/backend/value': 'Qt4Agg',
               'umd/enabled': True,
               'umd/verbose': True,
               'umd/namelist': ['guidata', 'guiqwt'],
               'light_background': True,
               'ipython_set_color': True,
-              'remove_pyqt_inputhook': os.name == 'nt',
-              'ignore_sip_setapi_errors': True,
               }),
             ('variable_explorer',
              {
               'shortcut': "Ctrl+Shift+V",
               'autorefresh': True,
               'autorefresh/timeout': 2000,
-              'editable_types': type2str(EDITABLE_TYPES),
-              'picklable_types': type2str(PICKLABLE_TYPES),
-              'itermax': ITERMAX,
-              'excluded_names': EXCLUDED,
+              'check_all': CHECK_ALL,
+              'excluded_names': EXCLUDED_NAMES,
               'exclude_private': True,
-              'exclude_upper': True,
+              'exclude_uppercase': True,
+              'exclude_capitalized': False,
               'exclude_unsupported': True,
               'inplace': False,
               'truncate': True,
               'minmax': False,
               'collvalue': False,
-              'remote_editing': True,
+              'remote_editing': False,
               }),
             ('editor',
              {
@@ -192,19 +229,22 @@ DEFAULTS = [
               'font/bold': False,
               'wrap': False,
               'wrapflag': True,
-              'code_analysis': True,
+              'code_analysis/pyflakes': True,
+              'code_analysis/pep8': False,
               'todo_list': True,
               'realtime_analysis': True,
               'realtime_analysis/timeout': 2500,
               'outline_explorer': True,
               'line_numbers': True,
               'edge_line': True,
-              'edge_line_column': 80,
+              'edge_line_column': 79,
               'toolbox_panel': True,
               'calltips': True,
               'go_to_definition': True,
               'close_parentheses': True,
               'auto_unindent': True,
+              'indent_chars': '*    *',
+              'tab_stop_width': 40,
               'object_inspector': True,
               'codecompletion/auto': True,
               'codecompletion/enter_key': True,
@@ -215,6 +255,8 @@ DEFAULTS = [
               'intelligent_backspace': True,
               'highlight_current_line': True,
               'occurence_highlighting': True,
+              'occurence_highlighting/timeout': 1500,
+              'always_remove_trailing_spaces': False,
               'fullpath_sorting': True,
               'show_tab_bar': True,
               'max_recent_files': 20,
@@ -229,6 +271,7 @@ DEFAULTS = [
               'font/italic': False,
               'font/bold': False,
               'wrap': True,
+              'go_to_eof': True,
               }),
             ('inspector',
              {
@@ -240,7 +283,7 @@ DEFAULTS = [
               'font/italic': False,
               'font/bold': False,
               'rich_text/font/family': SANS_SERIF,
-              'rich_text/font/size': 11,
+              'rich_text/font/size': BIG,
               'rich_text/font/italic': False,
               'rich_text/font/bold': False,
               'wrap': True,
@@ -253,10 +296,17 @@ DEFAULTS = [
               'zoom_factor': .8,
               'max_history_entries': 20,
               }),
+            ('outline_explorer',
+             {
+              'shortcut': "Ctrl+Shift+O",
+              'enable': True,
+              }),
             ('project_explorer',
              {
               'shortcut': "Ctrl+Shift+P",
               'enable': True,
+              'name_filters': NAME_FILTERS,
+              'show_all': False,
               }),
             ('arrayeditor',
              {
@@ -283,11 +333,7 @@ DEFAULTS = [
              {
               'enable': True,
               'wrap': True,
-              'name_filters': ['*.py', '*.pyw', '*.ipy', '*.pth',
-                               '*.npy', '*.mat', '*.spydata',
-                               '*.txt', '*.csv', '*.dat'],
-              'valid_filetypes': ['', '.py', '.pyw', '.ipy', '.spydata', '.npy',
-                                  '.pth', '.txt', '.csv', '.mat', '.h5'],
+              'name_filters': NAME_FILTERS,
               'show_hidden': True,
               'show_all': False,
               'show_toolbar': True,
@@ -297,13 +343,13 @@ DEFAULTS = [
              {
               'enable': True,
               'supported_encodings': ["utf-8", "iso-8859-1", "cp1252"],
-              'include': [r'\.pyw?$|\.ipy$|\.txt$|\.c$|\.cpp$|\.h$|\.hpp$|\.f$|\.ini$', '.'],
+              'include': INCLUDE_PATTERNS,
               'include_regexp': True,
-              'exclude': [r'\.pyc$|\.pyo$|\.orig$|\.hg|\.svn'],
+              'exclude': EXCLUDE_PATTERNS,
               'exclude_regexp': True,
               'search_text_regexp': True,
               'search_text': [''],
-              'search_text_samples': [r'# ?TODO|# ?FIXME|# ?XXX|# ?HINT|# ?TIP'],
+              'search_text_samples': [codeanalysis.TASKS_PATTERN],
               'in_python_path': False,
               'more_options': True,
               }),
@@ -311,34 +357,14 @@ DEFAULTS = [
 
 DEV = not __file__.startswith(sys.prefix)
 DEV = False
-from spyderlib import __version__
-_subfolder = '.spyder%s' % __version__.split('.')[0]
-CONF = UserConfig('spyder', defaults=DEFAULTS, load=(not DEV), version='2.0.6',
-                  subfolder=_subfolder, backup=True, raw_mode=True)
+CONF = UserConfig('spyder', defaults=DEFAULTS, load=(not DEV), version='2.2.0',
+                  subfolder=SUBFOLDER, backup=True, raw_mode=True)
 # Removing old .spyder.ini location:
 old_location = osp.join(get_home_dir(), '.spyder.ini')
 if osp.isfile(old_location):
     os.remove(old_location)
 
 
-# Translation support
-if CONF.get('main', 'translation'):
-    from spyderlib.utils.translations import get_translation
-    _ = get_translation("spyderlib")
-else:
-    _ = lambda x: x
-
-
-def get_conf_path(filename=None):
-    """Return absolute path for configuration file with specified filename"""
-    conf_dir = osp.join(get_home_dir(), _subfolder)
-    if not osp.isdir(conf_dir):
-        os.mkdir(conf_dir)
-    if filename is None:
-        return conf_dir
-    else:
-        return osp.join(conf_dir, filename)
-
 IMG_PATH = []
 def add_image_path(path):
     if not osp.isdir(path):
@@ -349,9 +375,14 @@ def add_image_path(path):
         for dir in dirs:
             IMG_PATH.append(osp.join(path, dir))
 
-add_image_path(osp.join(DATA_PATH, 'images'))
+add_image_path(get_module_data_path('spyderlib', relpath='images'))
 
-def get_image_path( name, default="not_found.png" ):
+from spyderlib.otherplugins import PLUGIN_PATH
+if PLUGIN_PATH is not None:
+    add_image_path(osp.join(PLUGIN_PATH, 'images'))
+
+
+def get_image_path(name, default="not_found.png"):
     """Return image absolute path"""
     for img_path in IMG_PATH:
         full_path = osp.join(img_path, name)
@@ -476,31 +507,3 @@ for _name in COLOR_SCHEME_NAMES:
     set_default_color_scheme(_name, replace=False)
 CUSTOM_COLOR_SCHEME_NAME = "Custom"
 set_color_scheme(CUSTOM_COLOR_SCHEME_NAME, COLORS["Spyder"], replace=False)
-
-
-PLUGIN_PATH = None
-from spyderlib.utils import programs
-if programs.is_module_installed("spyderplugins"):
-    import spyderplugins
-    PLUGIN_PATH = osp.abspath(spyderplugins.__path__[0])
-    add_image_path(osp.join(PLUGIN_PATH, 'images'))
-
-def get_spyderplugins(prefix, extension):
-    """Scan spyderplugins module and
-    return the list of module names matching *prefix* and *extension*"""
-    plist = []
-    if PLUGIN_PATH is not None:
-        for name in os.listdir(PLUGIN_PATH):
-            modname, ext = osp.splitext(name)
-            if prefix is not None and not name.startswith(prefix):
-                continue
-            if extension is not None and ext != extension:
-                continue
-            plist.append(modname)
-    return plist
-
-def get_spyderplugins_mods(prefix, extension):
-    """Scan spyderplugins module and
-    return the list of modules matching *prefix* and *extension*"""
-    return [getattr(__import__('spyderplugins.%s' % modname), modname)
-            for modname in get_spyderplugins(prefix, extension)]
