@@ -6,17 +6,36 @@
 
 """Patching matplotlib's FigureManager"""
 
+import sys
 
-def set_backend(backend):
-    """Set matplotlib's backend: Qt4Agg, WXAgg, ..."""
+
+def is_available():
+    """Is Matplotlib installed version supported by this patch?"""
     import matplotlib
-    matplotlib.use(backend)
+    mpl_ver = matplotlib.__version__.split('.')
+    if int(mpl_ver[0]) < 1 or int(mpl_ver[0]) == 1 and int(mpl_ver[1]) == 0:
+        # Matplotlib <=v1.0 is installed
+        return True
 
 
 def apply():
     """Monkey patching matplotlib Qt4 backend figures"""
+    if not is_available():
+        return
+
+    # Warning: do not move these import statements outside this function,
+    # otherwise, PyQt would be imported as soon as this module would be.
+    from spyderlib.qt import is_pyqt46
     from spyderlib.qt.QtGui import QIcon, QCursor, QInputDialog, QMainWindow
-    from spyderlib.qt.QtCore import Qt, PYQT_VERSION_STR, SIGNAL, QObject
+    from spyderlib.qt.QtCore import Qt, SIGNAL, QObject
+    
+    # Avoid using matplotlib's formlayout version which is not compatible 
+    # with PyQt4 API #2 and PySide (at least up to Matplotlib v1.0.1)
+    from spyderlib.widgets import formlayout
+    sys.modules['matplotlib.backends.qt4_editor.formlayout'] = formlayout
+    import matplotlib.backends.qt4_editor
+    matplotlib.backends.qt4_editor.formlayout = formlayout
+    
     from matplotlib.backends import backend_qt4
     
     # Class added to matplotlib to fix a bug with PyQt4 v4.6+
@@ -26,7 +45,7 @@ def apply():
             
         def closeEvent(self, event):
             super(FigureWindow, self).closeEvent(event)
-            if PYQT_VERSION_STR.startswith('4.6'):
+            if is_pyqt46:
                 self.emit(SIGNAL('destroyed()'))
     # ****************************************************************
     # *  FigureManagerQT
@@ -115,7 +134,7 @@ def apply():
                             text += ": "+ylabel
                         text += " (%s)"
                     elif ylabel:
-                        text = "%s (%s)" % ylabel
+                        text = ylabel+" (%s)"
                     else:
                         text = "%s"
                     titles.append(text % repr(axes))
