@@ -11,9 +11,10 @@
 # pylint: disable=R0911
 # pylint: disable=R0201
 
-from spyderlib.qt.QtGui import (QVBoxLayout, QFileDialog, QFontDialog,
-                                QInputDialog, QLineEdit, QMenu)
+from spyderlib.qt.QtGui import (QVBoxLayout, QFontDialog, QInputDialog,
+                                QLineEdit, QMenu)
 from spyderlib.qt.QtCore import SIGNAL
+from spyderlib.qt.compat import getopenfilename
 
 import os, sys
 import os.path as osp
@@ -23,8 +24,9 @@ STDOUT = sys.stdout
 
 
 # Local imports
-from spyderlib.config import get_icon, _
-from spyderlib.utils import get_error_match
+from spyderlib.baseconfig import _
+from spyderlib.config import get_icon
+from spyderlib.utils.misc import get_error_match
 from spyderlib.utils.qthelpers import (create_action, add_actions, mimedata2url,
                                        DialogManager)
 from spyderlib.utils.environ import EnvDialog
@@ -42,6 +44,7 @@ class Console(SpyderPluginWidget):
     def __init__(self, parent=None, namespace=None, commands=[], message="",
                  debug=False, exitfunc=None, profile=False,
                  multithreaded=False):
+        SpyderPluginWidget.__init__(self, parent)
         
         self.dialog_manager = DialogManager()
 
@@ -64,8 +67,9 @@ class Console(SpyderPluginWidget):
                      lambda state: self.emit(SIGNAL('redirect_stdio(bool)'),
                                              state))
         
-        SpyderPluginWidget.__init__(self, parent)
-        
+        # Initialize plugin
+        self.initialize_plugin()
+                
         # Find/replace widget
         self.find_widget = FindReplace(self)
         self.find_widget.set_editor(self.shell)
@@ -207,15 +211,12 @@ class Console(SpyderPluginWidget):
         """Run a Python script"""
         if filename is None:
             self.shell.interpreter.restore_stds()
-            filename = QFileDialog.getOpenFileName(self,
-                          _("Run Python script"), os.getcwdu(),
-                          _("Python scripts")+" (*.py ; *.pyw ; *.ipy)")
+            filename, _selfilter = getopenfilename(self, _("Run Python script"),
+                   os.getcwdu(), _("Python scripts")+" (*.py ; *.pyw ; *.ipy)")
             self.shell.interpreter.redirect_stds()
             if filename:
-                filename = unicode(filename)
                 os.chdir( os.path.dirname(filename) )
                 filename = os.path.basename(filename)
-                self.emit(SIGNAL("refresh()"))
             else:
                 return
         command = "runfile(%s, args=%s)" % (repr(osp.abspath(filename)),
