@@ -17,10 +17,10 @@ import sys, os
 STDOUT = sys.stdout
 
 from spyderlib.qt.QtGui import QApplication
-from spyderlib.qt.QtCore import SIGNAL
+from spyderlib.qt.QtCore import SIGNAL, Signal
 
 # Local imports
-from spyderlib.config import _
+from spyderlib.baseconfig import _
 from spyderlib.utils.qthelpers import create_action
 from spyderlib.widgets.findinfiles import FindInFilesWidget
 from spyderlib.plugins import SpyderPluginMixin
@@ -29,6 +29,7 @@ from spyderlib.plugins import SpyderPluginMixin
 class FindInFiles(FindInFilesWidget, SpyderPluginMixin):
     """Find in files DockWidget"""
     CONF_SECTION = 'find_in_files'
+    sig_option_changed = Signal(str, object)
     def __init__(self, parent=None):
         supported_encodings = self.get_option('supported_encodings')
         
@@ -56,6 +57,9 @@ class FindInFiles(FindInFilesWidget, SpyderPluginMixin):
                                    in_python_path, more_options)
         SpyderPluginMixin.__init__(self, parent)
         
+        # Initialize plugin
+        self.initialize_plugin()
+        
         self.connect(self, SIGNAL('toggle_visibility(bool)'), self.toggle)
         
     def toggle(self, state):
@@ -79,6 +83,8 @@ class FindInFiles(FindInFilesWidget, SpyderPluginMixin):
             if widget.has_selected_text():
                 text = widget.get_selected_text()
         self.set_search_text(text)
+        if text:
+            self.find()
         
     #------ SpyderPluginWidget API ---------------------------------------------    
     def get_plugin_title(self):
@@ -98,7 +104,7 @@ class FindInFiles(FindInFilesWidget, SpyderPluginMixin):
     
     def register_plugin(self):
         """Register plugin in Spyder's main window"""
-        self.set_pythonpath_callback(self.main.get_spyder_pythonpath)
+        self.get_pythonpath_callback = self.main.get_spyder_pythonpath
         self.main.add_dockwidget(self)
         self.connect(self, SIGNAL("edit_goto(QString,int,QString)"),
                      self.main.editor.load)
@@ -121,6 +127,7 @@ class FindInFiles(FindInFilesWidget, SpyderPluginMixin):
         
     def closing_plugin(self, cancelable=False):
         """Perform actions before parent main window is closed"""
+        self.closing_widget()  # stop search thread and clean-up
         options = self.find_options.get_options(all=True)
         if options is not None:
             search_text, text_re, search_path, \
