@@ -18,11 +18,12 @@ STDOUT = sys.stdout
 
 # Local imports
 from spyderlib.utils import encoding
-from spyderlib.config import get_icon, CONF, get_color_scheme, _
+from spyderlib.baseconfig import _
+from spyderlib.config import get_icon, CONF, get_color_scheme
 from spyderlib.utils.qthelpers import (create_action, create_toolbutton,
                                        add_actions)
 from spyderlib.widgets.tabs import Tabs
-from spyderlib.widgets.editor import CodeEditor
+from spyderlib.widgets.sourcecode import codeeditor
 from spyderlib.widgets.findreplace import FindReplace
 from spyderlib.plugins import SpyderPluginWidget, PluginConfigPage
 
@@ -37,6 +38,8 @@ class HistoryConfigPage(PluginConfigPage):
 
         sourcecode_group = QGroupBox(_("Source code"))
         wrap_mode_box = self.create_checkbox(_("Wrap lines"), 'wrap')
+        go_to_eof_box = self.create_checkbox(
+                        _("Scroll automatically to last entry"), 'go_to_eof')
         font_group = self.create_fontgroup(option=None,
                                     text=_("Font style"),
                                     fontfilters=QFontComboBox.MonospacedFonts)
@@ -51,6 +54,7 @@ class HistoryConfigPage(PluginConfigPage):
 
         sourcecode_layout = QVBoxLayout()
         sourcecode_layout.addWidget(wrap_mode_box)
+        sourcecode_layout.addWidget(go_to_eof_box)
         sourcecode_layout.addWidget(cs_combo)
         sourcecode_group.setLayout(sourcecode_layout)
         
@@ -80,6 +84,9 @@ class HistoryLog(SpyderPluginWidget):
         
         SpyderPluginWidget.__init__(self, parent)
 
+        # Initialize plugin
+        self.initialize_plugin()
+        
         self.set_default_color_scheme()
         
         layout = QVBoxLayout()
@@ -201,7 +208,7 @@ class HistoryLog(SpyderPluginWidget):
         filename = encoding.to_unicode(filename)
         if filename in self.filenames:
             return
-        editor = CodeEditor(self)
+        editor = codeeditor.CodeEditor(self)
         if osp.splitext(filename)[1] == '.py':
             language = 'py'
             icon = get_icon('python.png')
@@ -239,7 +246,8 @@ class HistoryLog(SpyderPluginWidget):
         filename, command = encoding.to_unicode(filename), unicode(command)
         index = self.filenames.index(filename)
         self.editors[index].append(command)
-        self.editors[index].set_cursor_position('eof')
+        if self.get_option('go_to_eof'):
+            self.editors[index].set_cursor_position('eof')
         self.tabwidget.setCurrentIndex(index)
         
     def change_history_depth(self):
