@@ -18,13 +18,14 @@ import sys, re, os.path as osp, socket
 STDOUT = sys.stdout
 
 # Local imports
-from spyderlib.config import (get_conf_path, get_icon, CONF, get_color_scheme,
-                              get_font, set_font, _)
+from spyderlib.baseconfig import get_conf_path, _
+from spyderlib.config import (get_icon, CONF, get_color_scheme,
+                              get_font, set_font)
 from spyderlib.utils import programs
 from spyderlib.utils.qthelpers import (create_toolbutton, add_actions,
                                        create_action)
 from spyderlib.widgets.comboboxes import EditableComboBox
-from spyderlib.widgets.editor import CodeEditor
+from spyderlib.widgets.sourcecode import codeeditor
 from spyderlib.widgets.findreplace import FindReplace
 from spyderlib.widgets.browser import WebView
 from spyderlib.widgets.externalshell.pythonshell import ExtPythonShellWidget
@@ -149,7 +150,7 @@ class PlainText(QWidget):
         self.editor = None
 
         # Read-only editor
-        self.editor = CodeEditor(self)
+        self.editor = codeeditor.CodeEditor(self)
         self.editor.setup_editor(linenumbers=False, language='py',
                                  scrollflagarea=False)
         self.connect(self.editor, SIGNAL("focus_changed()"),
@@ -199,6 +200,9 @@ class ObjectInspector(SpyderPluginWidget):
     def __init__(self, parent):
         SpyderPluginWidget.__init__(self, parent)
 
+        # Initialize plugin
+        self.initialize_plugin()
+
         self.no_doc_string = _("No documentation available")
         
         self._last_console_cb = None
@@ -322,7 +326,7 @@ class ObjectInspector(SpyderPluginWidget):
         layout.addWidget(self.rich_text)
         self.setLayout(layout)
         
-        QTimer.singleShot(8000, self.refresh_plugin)
+        self._starting_up = True
             
     #------ SpyderPluginWidget API ---------------------------------------------    
     def get_plugin_title(self):
@@ -359,6 +363,9 @@ class ObjectInspector(SpyderPluginWidget):
         
     def refresh_plugin(self):
         """Refresh widget"""
+        if self._starting_up:
+            self._starting_up = False
+            QTimer.singleShot(5000, self.refresh_plugin)
         self.set_object_text(None, force_refresh=False)
 
     def apply_plugin_settings(self, options):
@@ -532,7 +539,7 @@ class ObjectInspector(SpyderPluginWidget):
         
         if self.dockwidget is not None:
             self.dockwidget.blockSignals(True)
-        self.__eventually_raise_inspector(text)
+        self.__eventually_raise_inspector(text, force=force_refresh)
         if self.dockwidget is not None:
             self.dockwidget.blockSignals(False)
         
@@ -553,20 +560,22 @@ class ObjectInspector(SpyderPluginWidget):
         
         if self.dockwidget is not None:
             self.dockwidget.blockSignals(True)
-        self.__eventually_raise_inspector(doctxt)
+        self.__eventually_raise_inspector(doctxt, force=force_refresh)
         if self.dockwidget is not None:
             self.dockwidget.blockSignals(False)
             
-    def __eventually_raise_inspector(self, text):
+    def __eventually_raise_inspector(self, text, force=False):
         index = self.source_combo.currentIndex()
         if hasattr(self.main, 'tabifiedDockWidgets'):
             # 'QMainWindow.tabifiedDockWidgets' was introduced in PyQt 4.5
-            if self.dockwidget and self.dockwidget.isVisible() \
-               and not self.ismaximized and text != self._last_texts[index]:
+            if self.dockwidget and (force or self.dockwidget.isVisible()) \
+               and not self.ismaximized \
+               and (force or text != self._last_texts[index]):
                 dockwidgets = self.main.tabifiedDockWidgets(self.dockwidget)
                 if self.main.console.dockwidget not in dockwidgets and \
                    (hasattr(self.main, 'extconsole') and \
                     self.main.extconsole.dockwidget not in dockwidgets):
+                    self.dockwidget.show()
                     self.dockwidget.raise_()
         self._last_texts[index] = text
     
