@@ -6,13 +6,15 @@
 
 """Online Help Plugin"""
 
+from spyderlib.qt.QtCore import Signal
+
 import sys, os.path as osp
 
 # For debugging purpose:
 STDOUT = sys.stdout
 
 # Local imports
-from spyderlib.config import get_conf_path, _
+from spyderlib.baseconfig import get_conf_path, _
 from spyderlib.widgets.pydocgui import PydocBrowser
 from spyderlib.plugins import SpyderPluginMixin
 
@@ -21,12 +23,18 @@ class OnlineHelp(PydocBrowser, SpyderPluginMixin):
     """
     Online Help Plugin
     """
+    sig_option_changed = Signal(str, object)
     CONF_SECTION = 'onlinehelp'
     LOG_PATH = get_conf_path('.onlinehelp')
     def __init__(self, parent):
         self.main = parent
         PydocBrowser.__init__(self, parent)
         SpyderPluginMixin.__init__(self, parent)
+
+        # Initialize plugin
+        self.initialize_plugin()
+
+        self.register_widget_shortcuts("Editor", self.find_widget)
         
         self.webview.set_zoom_factor(self.get_option('zoom_factor'))
         self.url_combo.setMaxCount(self.get_option('max_history_entries'))
