@@ -6,25 +6,29 @@
 
 """Run configurations related dialogs and widgets and data models"""
 
-from spyderlib.qt.QtGui import (QVBoxLayout, QFileDialog, QDialog, QWidget,
-                                QGroupBox, QLabel, QPushButton, QCheckBox,
-                                QLineEdit, QComboBox, QHBoxLayout,
-                                QDialogButtonBox, QStackedWidget, QGridLayout,
-                                QSizePolicy, QRadioButton, QMessageBox)
-from spyderlib.qt.QtCore import SIGNAL, SLOT
+from spyderlib.qt.QtGui import (QVBoxLayout, QDialog, QWidget, QGroupBox,
+                                QLabel, QPushButton, QCheckBox, QLineEdit,
+                                QComboBox, QHBoxLayout, QDialogButtonBox,
+                                QStackedWidget, QGridLayout, QSizePolicy,
+                                QRadioButton, QMessageBox)
+from spyderlib.qt.QtCore import SIGNAL, SLOT, Qt
+from spyderlib.qt.compat import getexistingdirectory
 
-import os, sys
+import os
+import sys
 import os.path as osp
 
 # For debugging purpose:
 STDOUT = sys.stdout
 
 # Local imports
-from spyderlib.config import get_icon, CONF, _
+from spyderlib.baseconfig import _
+from spyderlib.config import get_icon, CONF
 from spyderlib.utils.qthelpers import get_std_icon
 
 
 class RunConfiguration(object):
+    """Run configuration"""
     def __init__(self, fname=None):
         self.args = ''
         self.args_enabled = False
@@ -34,6 +38,7 @@ class RunConfiguration(object):
             self.wdir = osp.dirname(fname)
             self.wdir_enabled = True
         self.current = False
+        self.systerm = False
         self.interact = False
         self.python_args = ''
         self.python_args_enabled = False
@@ -44,6 +49,7 @@ class RunConfiguration(object):
         self.wdir = options.get('workdir', os.getcwdu())
         self.wdir_enabled = options.get('workdir/enabled', False)
         self.current = options.get('current', False)
+        self.systerm = options.get('systerm', False)
         self.interact = options.get('interact', False)
         self.python_args = options.get('python_args', '')
         self.python_args_enabled = options.get('python_args/enabled', False)
@@ -55,6 +61,7 @@ class RunConfiguration(object):
                 'workdir/enabled': self.wdir_enabled,
                 'workdir': self.wdir,
                 'current': self.current,
+                'systerm': self.systerm,
                 'interact': self.interact,
                 'python_args/enabled': self.python_args_enabled,
                 'python_args': self.python_args,
@@ -81,9 +88,13 @@ class RunConfiguration(object):
         
 def _get_run_configurations():
     history_count = CONF.get('run', 'history', 20)
-    return [(filename, options)
-            for filename, options in CONF.get('run', 'configurations', [])
-            if osp.isfile(filename)][:history_count]
+    try:
+        return [(filename, options)
+                for filename, options in CONF.get('run', 'configurations', [])
+                if osp.isfile(filename)][:history_count]
+    except ValueError:
+        CONF.set('run', 'configurations', [])
+        return []
 
 def _set_run_configurations(configurations):
     history_count = CONF.get('run', 'history', 20)
@@ -100,6 +111,7 @@ def get_run_configuration(fname):
 
 
 class RunConfigOptions(QWidget):
+    """Run configuration options"""
     def __init__(self, parent=None):
         QWidget.__init__(self, parent)
         self.runconf = RunConfiguration()
@@ -132,11 +144,14 @@ class RunConfigOptions(QWidget):
         radio_layout = QVBoxLayout()
         radio_group.setLayout(radio_layout)
         self.current_radio = QRadioButton(_("Execute in current Python "
-                                                  "or IPython interpreter"))
+                                            "or IPython interpreter"))
         radio_layout.addWidget(self.current_radio)
         self.new_radio = QRadioButton(_("Execute in a new dedicated "
-                                              "Python interpreter"))
+                                        "Python interpreter"))
         radio_layout.addWidget(self.new_radio)
+        self.systerm_radio = QRadioButton(_("Execute in an external "
+                                            "system terminal"))
+        radio_layout.addWidget(self.systerm_radio)
         
         new_group = QGroupBox(_("Dedicated Python interpreter"))
         self.connect(self.current_radio, SIGNAL("toggled(bool)"),
@@ -144,7 +159,7 @@ class RunConfigOptions(QWidget):
         new_layout = QGridLayout()
         new_group.setLayout(new_layout)
         self.interact_cb = QCheckBox(_("Interact with the Python "
-                                             "interpreter after execution"))
+                                       "interpreter after execution"))
         new_layout.addWidget(self.interact_cb, 1, 0, 1, -1)
         self.pclo_cb = QCheckBox(_("Command line options:"))
         new_layout.addWidget(self.pclo_cb, 2, 0)
@@ -154,7 +169,7 @@ class RunConfigOptions(QWidget):
         self.pclo_edit.setEnabled(False)
         new_layout.addWidget(self.pclo_edit, 2, 1)
         pclo_label = QLabel(_("The <b>-u</b> option is "
-                                    "added to these commands"))
+                              "added to these commands"))
         pclo_label.setWordWrap(True)
         new_layout.addWidget(pclo_label, 3, 1)
         
@@ -171,9 +186,8 @@ class RunConfigOptions(QWidget):
         basedir = unicode(self.wd_edit.text())
         if not osp.isdir(basedir):
             basedir = os.getcwdu()
-        directory = QFileDialog.getExistingDirectory(self,
-                                        _("Select directory"), basedir)
-        if not directory.isEmpty():
+        directory = getexistingdirectory(self, _("Select directory"), basedir)
+        if directory:
             self.wd_edit.setText(directory)
             self.wd_cb.setChecked(True)
         
@@ -185,6 +199,8 @@ class RunConfigOptions(QWidget):
         self.wd_edit.setText(self.runconf.wdir)
         if self.runconf.current:
             self.current_radio.setChecked(True)
+        elif self.runconf.systerm:
+            self.systerm_radio.setChecked(True)
         else:
             self.new_radio.setChecked(True)
         self.interact_cb.setChecked(self.runconf.interact)
@@ -197,6 +213,7 @@ class RunConfigOptions(QWidget):
         self.runconf.wdir_enabled = self.wd_cb.isChecked()
         self.runconf.wdir = unicode(self.wd_edit.text())
         self.runconf.current = self.current_radio.isChecked()
+        self.runconf.systerm = self.systerm_radio.isChecked()
         self.runconf.interact = self.interact_cb.isChecked()
         self.runconf.python_args_enabled = self.pclo_cb.isChecked()
         self.runconf.python_args = unicode(self.pclo_edit.text())
@@ -213,9 +230,29 @@ class RunConfigOptions(QWidget):
             return False
 
 
-class RunConfigOneDialog(QDialog):
+class SizeMixin(object):
+    """Mixin to keep widget size accessible
+    even when C++ object has been deleted"""
+    def __init__(self):
+        self.win_size = None
+
+    def resizeEvent(self, event):
+        """Reimplement Qt method"""
+        QDialog.resizeEvent(self, event)
+        self.win_size = self.size()
+
+
+class RunConfigOneDialog(QDialog, SizeMixin):
+    """Run configuration dialog box: single file version"""
     def __init__(self, parent=None):
         QDialog.__init__(self, parent)
+        SizeMixin.__init__(self)
+        
+        # Destroying the C++ object right after closing the dialog box,
+        # otherwise it may be garbage-collected in another QThread
+        # (e.g. the editor's analysis thread in Spyder), thus leading to
+        # a segmentation fault on UNIX or an application crash on Windows
+        self.setAttribute(Qt.WA_DeleteOnClose)
         
         self.filename = None
         
@@ -252,12 +289,22 @@ class RunConfigOneDialog(QDialog):
         QDialog.accept(self)
     
     def get_configuration(self):
+        # It is import to avoid accessing Qt C++ object as it has probably
+        # already been destroyed, due to the Qt.WA_DeleteOnClose attribute
         return self.runconfigoptions.runconf
 
 
-class RunConfigDialog(QDialog):
+class RunConfigDialog(QDialog, SizeMixin):
+    """Run configuration dialog box: multiple file version"""
     def __init__(self, parent=None):
         QDialog.__init__(self, parent)
+        SizeMixin.__init__(self)
+        
+        # Destroying the C++ object right after closing the dialog box,
+        # otherwise it may be garbage-collected in another QThread
+        # (e.g. the editor's analysis thread in Spyder), thus leading to
+        # a segmentation fault on UNIX or an application crash on Windows
+        self.setAttribute(Qt.WA_DeleteOnClose)
         
         self.file_to_run = None
         
