@@ -9,13 +9,14 @@
 import sys
 
 from spyderlib.qt.QtGui import QStackedWidget, QGroupBox, QVBoxLayout
-from spyderlib.qt.QtCore import SIGNAL
+from spyderlib.qt.QtCore import Signal
 
 # For debugging purpose:
 STDOUT = sys.stdout
 
 # Local imports
-from spyderlib.config import CONF, get_icon, _
+from spyderlib.baseconfig import _
+from spyderlib.config import CONF, get_icon
 from spyderlib.utils import programs
 from spyderlib.plugins import SpyderPluginMixin, PluginConfigPage
 from spyderlib.widgets.externalshell.monitor import REMOTE_SETTINGS
@@ -34,7 +35,8 @@ class VariableExplorerConfigPage(PluginConfigPage):
         filter_group = QGroupBox(_("Filter"))
         filter_data = [
             ('exclude_private', _("Exclude private references")),
-            ('exclude_upper', _("Exclude capitalized references")),
+            ('exclude_capitalized', _("Exclude capitalized references")),
+            ('exclude_uppercase', _("Exclude all-uppercase references")),
             ('exclude_unsupported', _("Exclude unsupported data types")),
                 ]
         filter_boxes = [self.create_checkbox(text, option)
@@ -87,11 +89,15 @@ class VariableExplorer(QStackedWidget, SpyderPluginMixin):
     """
     CONF_SECTION = 'variable_explorer'
     CONFIGWIDGET_CLASS = VariableExplorerConfigPage
+    sig_option_changed = Signal(str, object)
     def __init__(self, parent):
         QStackedWidget.__init__(self, parent)
         SpyderPluginMixin.__init__(self, parent)
         self.shellwidgets = {}
 
+        # Initialize plugin
+        self.initialize_plugin()
+
     @staticmethod
     def get_settings():
         """
@@ -116,9 +122,7 @@ class VariableExplorer(QStackedWidget, SpyderPluginMixin):
             nsb = NamespaceBrowser(self)
             nsb.set_shellwidget(shellwidget)
             nsb.setup(**VariableExplorer.get_settings())
-            self.connect(nsb, SIGNAL('option_changed'),
-                         lambda option, value:
-                         self.emit(SIGNAL('option_changed'), option, value))
+            nsb.sig_option_changed.connect(self.sig_option_changed.emit)
             self.addWidget(nsb)
             self.shellwidgets[shellwidget_id] = nsb
             self.set_shellwidget(shellwidget)
@@ -189,12 +193,6 @@ class VariableExplorer(QStackedWidget, SpyderPluginMixin):
         """Register plugin in Spyder's main window"""
         self.main.extconsole.set_variableexplorer(self)
         self.main.add_dockwidget(self)
-        if self.main.explorer is not None:
-            self.connect(self.main.explorer, SIGNAL("import_data(QString)"),
-                         self.import_data)
-        if self.main.projectexplorer is not None:
-            self.connect(self.main.projectexplorer,
-                         SIGNAL("import_data(QString)"), self.import_data)
         
     def apply_plugin_settings(self, options):
         """Apply configuration file's plugin settings"""
@@ -202,4 +200,4 @@ class VariableExplorer(QStackedWidget, SpyderPluginMixin):
             nsb.setup(**VariableExplorer.get_settings())
         ar_timeout = self.get_option('autorefresh/timeout')
         for shellwidget in self.main.extconsole.shellwidgets:
-            shellwidget.set_autorefresh_timeout(ar_timeout)
+            shellwidget.set_autorefresh_timeout(ar_timeout)
\ No newline at end of file
