@@ -8,14 +8,14 @@
 Module checking Spyder requirements
 """
 
-def check_requirement(package, module_name, version_attr, required_str):
+def check_version(package, module_name, version_attr, required_str):
     wng = "\n%s v%s or higher is required" % (package, required_str)
     try:
         module = __import__(module_name)
     except ImportError:
         return wng+" (not found!)"
     else:
-        if module_name.find('.'):
+        if '.' in module_name:
             module = getattr(module, module_name.split('.')[1])
         actual_str = getattr(module, version_attr)
         actual = actual_str.split('.')
@@ -46,10 +46,9 @@ def check_path():
                      "or directory '%s' must be in PYTHONPATH "
                      "environment variable" % dirname)
 
-def check_pyqt():
-    wng = check_requirement("PyQt", "PyQt4.QtCore", "PYQT_VERSION_STR", "4.4")
-    if wng:
-        show_warning("Please check Spyder installation requirements:"+wng)
-
-check_pyqt()
-check_path()
+def check_qt():
+    wng1 = check_version("PyQt", "PyQt4.QtCore", "PYQT_VERSION_STR", "4.4")
+    wng2 = check_version("PySide", "PySide", "__version__", "1.0")
+    if wng1 and wng2:
+        show_warning("Please check Spyder installation requirements:\n"
+                     +wng1+"\nor"+wng2)
