@@ -33,11 +33,12 @@ FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 """
 
-__version__ = '1.0.12'
+__version__ = '1.0.13'
 __license__ = __doc__
 
 import os, re, os.path as osp, shutil
-from ConfigParser import ConfigParser, MissingSectionHeaderError
+from ConfigParser import (ConfigParser, MissingSectionHeaderError,
+                          NoSectionError, NoOptionError)
 
 
 def get_home_dir():
@@ -72,19 +73,18 @@ class UserConfig(ConfigParser):
     Note that 'get' and 'set' arguments number and type
     differ from the overriden methods
     """
-    
-    default_section_name = 'main'
-    
+    DEFAULT_SECTION_NAME = 'main'
     def __init__(self, name, defaults=None, load=True, version=None,
-                 subfolder=None, backup=False, raw_mode=False):
+                 subfolder=None, backup=False, raw_mode=False,
+                 remove_obsolete=False):
         ConfigParser.__init__(self)
         self.raw = 1 if raw_mode else 0
         self.subfolder = subfolder
         if (version is not None) and (re.match('^(\d+).(\d+).(\d+)$', version) is None):
-            raise RuntimeError("Version number %r is incorrect - must be in X.Y.Z format" % version)
+            raise ValueError("Version number %r is incorrect - must be in X.Y.Z format" % version)
         self.name = name
         if isinstance(defaults, dict):
-            defaults = [ (self.default_section_name, defaults) ]
+            defaults = [ (self.DEFAULT_SECTION_NAME, defaults) ]
         self.defaults = defaults
         if defaults is not None:
             self.reset_to_defaults(save=False)
@@ -98,7 +98,10 @@ class UserConfig(ConfigParser):
             # If config file already exists, it overrides Default options:
             self.load_from_ini()
             old_ver = self.get_version(version)
-            if version != old_ver:
+            _major = lambda _t: _t[:_t.find('.')]
+            _minor = lambda _t: _t[:_t.rfind('.')]
+            # Resetting to defaults only if major/minor version is different
+            if _minor(version) != _minor(old_ver):
                 if backup:
                     try:
                         shutil.copyfile(fname, "%s-%s.bak" % (fname, old_ver))
@@ -106,7 +109,8 @@ class UserConfig(ConfigParser):
                         pass
                 # Version has changed -> overwriting .ini file
                 self.reset_to_defaults(save=False)
-                self.__remove_deprecated_options()
+                if remove_obsolete or _major(version) != _major(old_ver):
+                    self.__remove_deprecated_options()
                 # Set new version number
                 self.set_version(version, save=False)
             if defaults is None:
@@ -117,11 +121,11 @@ class UserConfig(ConfigParser):
         
     def get_version(self, version='0.0.0'):
         """Return configuration (not application!) version"""
-        return self.get(self.default_section_name, 'version', version)
+        return self.get(self.DEFAULT_SECTION_NAME, 'version', version)
         
     def set_version(self, version='0.0.0', save=True):
         """Set configuration (not application!) version"""
-        self.set(self.default_section_name, 'version', version, save=save)
+        self.set(self.DEFAULT_SECTION_NAME, 'version', version, save=save)
 
     def load_from_ini(self):
         """
@@ -198,11 +202,11 @@ class UserConfig(ConfigParser):
         Private method to check section and option types
         """
         if section is None:
-            section = self.default_section_name
+            section = self.DEFAULT_SECTION_NAME
         elif not isinstance(section, (str, unicode)):
-            raise RuntimeError, "Argument 'section' must be a string"
+            raise ValueError, "Argument 'section' must be a string"
         if not isinstance(option, (str, unicode)):
-            raise RuntimeError, "Argument 'option' must be a string"
+            raise ValueError, "Argument 'option' must be a string"
         return section
 
     def get_default(self, section, option):
@@ -229,13 +233,13 @@ class UserConfig(ConfigParser):
 
         if not self.has_section(section):
             if default is NoDefault:
-                raise RuntimeError("Unknown section %r" % section)
+                raise NoSectionError(section)
             else:
                 self.add_section(section)
         
         if not self.has_option(section, option):
             if default is NoDefault:
-                raise RuntimeError("Unknown option %r" % option)
+                raise NoOptionError(option, section)
             else:
                 self.set(section, option, default)
                 return default
