@@ -71,7 +71,7 @@ except ImportError:
 
 
 try:
-    import PIL.Image
+    from spyderlib.pil_patch import Image
     if sys.byteorder == 'little':
         _ENDIAN = '<'
     else:
@@ -91,7 +91,7 @@ try:
               "YCbCr": ('|u1', 4),
               }
     def __image_to_array(filename):
-        img = PIL.Image.open(filename)
+        img = Image.open(filename)
         try:
             dtype, extra = DTYPES[img.mode]
         except KeyError:
@@ -194,7 +194,7 @@ def load_dictionary(filename):
     return data, error_message
 
 
-from spyderlib.config import get_conf_path, get_spyderplugins_mods
+from spyderlib.baseconfig import get_conf_path
 
 SAVED_CONFIG_FILES = ('.inspector', '.onlinehelp', '.path', '.pylint.results',
                       '.spyder.ini', '.temp.py', '.temp.spydata', 'template.py',
@@ -283,13 +283,12 @@ def load_session(filename):
     return error_message
 
 
-from spyderlib.qt.QtGui import QFileDialog
-from spyderlib.qt.QtCore import QObject
-from spyderlib.config import _
+from spyderlib.baseconfig import _
 
-class IOFunctions(QObject):
+class IOFunctions(object):
     def __init__(self):
-        QObject.__init__(self)
+        self.load_extensions = None
+        self.save_extensions = None
         self.load_filters = None
         self.save_filters = None
         self.load_funcs = None
@@ -297,6 +296,8 @@ class IOFunctions(QObject):
         
     def setup(self):
         iofuncs = self.get_internal_funcs()+self.get_3rd_party_funcs()
+        load_extensions = {}
+        save_extensions = {}
         load_funcs = {}
         save_funcs = {}
         load_filters = []
@@ -306,9 +307,11 @@ class IOFunctions(QObject):
             filter_str = unicode(name + " (*%s)" % ext)
             if loadfunc is not None:
                 load_filters.append(filter_str)
+                load_extensions[filter_str] = ext
                 load_funcs[ext] = loadfunc
                 load_ext.append(ext)
             if savefunc is not None:
+                save_extensions[filter_str] = ext
                 save_filters.append(filter_str)
                 save_funcs[ext] = savefunc
         load_filters.insert(0, unicode(_("Supported files")+" (*"+\
@@ -317,6 +320,8 @@ class IOFunctions(QObject):
         self.save_filters = "\n".join(save_filters)
         self.load_funcs = load_funcs
         self.save_funcs = save_funcs
+        self.load_extensions = load_extensions
+        self.save_extensions = save_extensions
         
     def get_internal_funcs(self):
         return [
@@ -334,6 +339,7 @@ class IOFunctions(QObject):
         
     def get_3rd_party_funcs(self):
         other_funcs = []
+        from spyderlib.otherplugins import get_spyderplugins_mods
         for mod in get_spyderplugins_mods(prefix='io_', extension='.py'):
             try:
                 other_funcs.append((mod.FORMAT_EXT, mod.FORMAT_NAME,
@@ -341,14 +347,6 @@ class IOFunctions(QObject):
             except AttributeError, error:
                 print >>STDERR, "%s: %s" % (mod, str(error))
         return other_funcs
-    
-    def get_open_filenames(self, parent, filename, title):
-        return QFileDialog.getOpenFileNames(parent, title, filename,
-                                            self.load_filters)
-    
-    def get_save_filename(self, parent, filename, title):
-        return QFileDialog.getSaveFileName(parent, title, filename,
-                                           self.save_filters)
         
     def save(self, data, filename):
         ext = osp.splitext(filename)[1].lower()
