@@ -19,14 +19,16 @@ from spyderlib.qt.QtGui import (QHBoxLayout, QColor, QTableView, QItemDelegate,
                                 QMessageBox, QPushButton, QInputDialog, QMenu,
                                 QApplication, QKeySequence, QLabel, QComboBox,
                                 QStackedWidget, QWidget, QVBoxLayout)
-from spyderlib.qt.QtCore import (Qt, QVariant, QModelIndex,
-                                 QAbstractTableModel, SIGNAL, SLOT)
+from spyderlib.qt.QtCore import (Qt, QModelIndex, QAbstractTableModel, SIGNAL,
+                                 SLOT)
+from spyderlib.qt.compat import to_qvariant, from_qvariant
 
 import numpy as np
 import StringIO
 
 # Local imports
-from spyderlib.config import get_icon, get_font, _
+from spyderlib.baseconfig import _
+from spyderlib.config import get_icon, get_font
 from spyderlib.utils.qthelpers import (add_actions, create_action, keybinding,
                                        qapplication)
 
@@ -167,20 +169,20 @@ class ArrayModel(QAbstractTableModel):
     def data(self, index, role=Qt.DisplayRole):
         """Cell content"""
         if not index.isValid():
-            return QVariant()
+            return to_qvariant()
         value = self.get_value(index)
         if role == Qt.DisplayRole:
-            return QVariant(self._format % value)
+            return to_qvariant(self._format % value)
         elif role == Qt.TextAlignmentRole:
-            return QVariant(int(Qt.AlignCenter|Qt.AlignVCenter))
+            return to_qvariant(int(Qt.AlignCenter|Qt.AlignVCenter))
         elif role == Qt.BackgroundColorRole and self.bgcolor_enabled:
             hue = self.hue0+self.dhue*(self.vmax-self.color_func(value))/(self.vmax-self.vmin)
             hue = float(np.abs(hue))
             color = QColor.fromHsvF(hue, self.sat, self.val, self.alp)
-            return QVariant(color)
+            return to_qvariant(color)
         elif role == Qt.FontRole:
-            return QVariant(get_font('arrayeditor'))
-        return QVariant()
+            return to_qvariant(get_font('arrayeditor'))
+        return to_qvariant()
 
     def setData(self, index, value, role=Qt.EditRole):
         """Cell content change"""
@@ -188,7 +190,7 @@ class ArrayModel(QAbstractTableModel):
             return False
         i = index.row()
         j = index.column()
-        value = str(value.toString())
+        value = from_qvariant(value, str)
         if self._data.dtype.name == "bool":
             try:
                 val = bool(float(value))
@@ -237,12 +239,12 @@ class ArrayModel(QAbstractTableModel):
     def headerData(self, section, orientation, role=Qt.DisplayRole):
         """Set header data"""
         if role != Qt.DisplayRole:
-            return QVariant()
+            return to_qvariant()
         labels = self.xlabels if orientation == Qt.Horizontal else self.ylabels
         if labels is None:
-            return QVariant(int(section))
+            return to_qvariant(int(section))
         else:
-            return QVariant(labels[section])
+            return to_qvariant(labels[section])
 
 
 class ArrayDelegate(QItemDelegate):
@@ -256,7 +258,7 @@ class ArrayDelegate(QItemDelegate):
         model = index.model()
         if model._data.dtype.name == "bool":
             value = not model.get_value(index)
-            model.setData(index, QVariant(value))
+            model.setData(index, to_qvariant(value))
             return
         else:
             editor = QLineEdit(parent)
@@ -276,7 +278,7 @@ class ArrayDelegate(QItemDelegate):
 
     def setEditorData(self, editor, index):
         """Set editor widget's data"""
-        text = index.model().data(index, Qt.DisplayRole).toString()
+        text = from_qvariant(index.model().data(index, Qt.DisplayRole), str)
         editor.setText(text)
 
 
@@ -331,7 +333,6 @@ class ArrayView(QTableView):
         """Reimplement Qt method"""
         if event == QKeySequence.Copy:
             self.copy()
-            event.accept()
         else:
             QTableView.keyPressEvent(self, event)
 
@@ -425,6 +426,13 @@ class ArrayEditor(QDialog):
     """Array Editor Dialog"""    
     def __init__(self, parent=None):
         QDialog.__init__(self, parent)
+        
+        # Destroying the C++ object right after closing the dialog box,
+        # otherwise it may be garbage-collected in another QThread
+        # (e.g. the editor's analysis thread in Spyder), thus leading to
+        # a segmentation fault on UNIX or an application crash on Windows
+        self.setAttribute(Qt.WA_DeleteOnClose)
+        
         self.data = None
     
     def setup_and_check(self, data, title='', readonly=False,
@@ -529,6 +537,8 @@ class ArrayEditor(QDialog):
         
     def get_value(self):
         """Return modified array -- this is *not* a copy"""
+        # It is import to avoid accessing Qt C++ object as it has probably
+        # already been destroyed, due to the Qt.WA_DeleteOnClose attribute
         return self.data
 
     def error(self, message):
@@ -545,48 +555,49 @@ class ArrayEditor(QDialog):
         QDialog.reject(self)
     
     
-def aedit(data, title="", xlabels=None, ylabels=None,
-          readonly=False, parent=None):
-    """
-    Edit the array 'data' with the ArrayEditor and return the edited copy
-    (if Cancel is pressed, return None)
-    (instantiate a new QApplication if necessary,
-    so it can be called directly from the interpreter)
-    """
-    app = qapplication()
-    dialog = ArrayEditor(parent)
-    if dialog.setup_and_check(data, title, xlabels=xlabels, ylabels=ylabels,
-                              readonly=readonly):
-        dialog.show()
-        app.exec_()
-        if dialog.result():
-            return dialog.get_value()
+def test_edit(data, title="", xlabels=None, ylabels=None,
+              readonly=False, parent=None):
+    """Test subroutine"""
+    dlg = ArrayEditor(parent)
+    if dlg.setup_and_check(data, title, xlabels=xlabels, ylabels=ylabels,
+                           readonly=readonly) and dlg.exec_():
+        return dlg.get_value()
+    else:
+        import sys
+        sys.exit()
 
 
 def test():
     """Array editor test"""
+    _app = qapplication()
+    
     arr = np.array(["kjrekrjkejr"])
-    print "out:", aedit(arr, "string array")
+    print "out:", test_edit(arr, "string array")
     arr = np.array([u"kjrekrjkejr"])
-    print "out:", aedit(arr, "unicode array")
+    print "out:", test_edit(arr, "unicode array")
     arr = np.zeros((2,2), {'names': ('red', 'green', 'blue'),
                            'formats': (np.float32, np.float32, np.float32)})
-    print "out:", aedit(arr, "record array")
+    print "out:", test_edit(arr, "record array")
     arr = np.array([(0, 0.0), (0, 0.0), (0, 0.0)],
-                   dtype=[(('title 1', 'x'), '|i1'), (('title 2', 'y'), '>f4')])
-    print "out:", aedit(arr, "record array with titles")
+                   dtype=[(('title 1', 'x'), '|i1'),
+                          (('title 2', 'y'), '>f4')])
+    print "out:", test_edit(arr, "record array with titles")
     arr = np.random.rand(5, 5)
-    print "out:", aedit(arr, "float array", xlabels=['a', 'b', 'c', 'd', 'e'])
-    arr = np.round(np.random.rand(5, 5)*10)+np.round(np.random.rand(5, 5)*10)*1j
-    print "out:", aedit(arr, "complex array", xlabels=np.linspace(-12, 12, 5),
-                        ylabels=np.linspace(-12, 12, 5))
+    print "out:", test_edit(arr, "float array",
+                            xlabels=['a', 'b', 'c', 'd', 'e'])
+    arr = np.round(np.random.rand(5, 5)*10)+\
+                   np.round(np.random.rand(5, 5)*10)*1j
+    print "out:", test_edit(arr, "complex array",
+                            xlabels=np.linspace(-12, 12, 5),
+                            ylabels=np.linspace(-12, 12, 5))
     arr_in = np.array([True, False, True])
     print "in:", arr_in
-    arr_out = aedit(arr_in, "bool array")
+    arr_out = test_edit(arr_in, "bool array")
     print "out:", arr_out
     print arr_in is arr_out
     arr = np.array([1, 2, 3], dtype="int8")
-    print "out:", aedit(arr, "int array")
+    print "out:", test_edit(arr, "int array")
+
 
 if __name__ == "__main__":
     test()
