@@ -19,7 +19,7 @@ import sys
 import os.path as osp
 
 # Local imports
-from spyderlib.config import _
+from spyderlib.baseconfig import _
 
 # For debugging purpose:
 STDOUT = sys.stdout
@@ -51,16 +51,25 @@ class BaseComboBox(QComboBox):
             self.removeItem(index)
             index = self.findText(text)
         self.insertItem(0, text)
-        self.setCurrentIndex(0)
+        index = self.findText('')
+        if index != -1:
+            self.removeItem(index)
+            self.insertItem(0, '')
+            self.setCurrentIndex(1)
+        else:
+            self.setCurrentIndex(0)
+            
+    def add_current_text(self):
+        """Add current text to combo box history (convenient method)"""
+        self.add_text(self.currentText())
+        self.selected()
 
     def keyPressEvent(self, event):
         """Handle key press events"""
         if event.key() == Qt.Key_Return or event.key() == Qt.Key_Enter:
-            text = self.currentText()
-            valid = self.is_valid(text)
+            valid = self.is_valid(self.currentText())
             if valid or valid is None:
-                self.add_text(text)
-                self.selected()
+                self.add_current_text()
         else:
             QComboBox.keyPressEvent(self, event)
         
