@@ -16,134 +16,55 @@ Dictionary Editor Widget and Dialog based on Qt
 # pylint: disable=R0911
 # pylint: disable=R0201
 
-import re, os, sys
+import os, sys, datetime
 from spyderlib.qt.QtGui import (QMessageBox, QTableView, QItemDelegate,
                                 QLineEdit, QVBoxLayout, QWidget, QColor,
                                 QDialog, QDateEdit, QDialogButtonBox, QMenu,
                                 QInputDialog, QDateTimeEdit, QApplication,
-                                QKeySequence, QFileDialog)
-from spyderlib.qt.QtCore import (Qt, QVariant, QModelIndex,
-                                 QAbstractTableModel, SIGNAL, SLOT, QDateTime)
+                                QKeySequence)
+from spyderlib.qt.QtCore import (Qt, QModelIndex, QAbstractTableModel, SIGNAL,
+                                 SLOT, QDateTime, Signal)
+from spyderlib.qt.compat import to_qvariant, from_qvariant, getsavefilename
 
 # Local import
-from spyderlib.config import get_icon, get_font, _
-from spyderlib.utils import fix_reference_name
+from spyderlib.baseconfig import _
+from spyderlib.config import get_icon, get_font
+from spyderlib.utils.misc import fix_reference_name
 from spyderlib.utils.qthelpers import add_actions, create_action, qapplication
+from spyderlib.widgets.dicteditorutils import (sort_against, get_size,
+                   get_human_readable_type, value_to_display, get_color_name,
+                   is_known_type, FakeObject, Image, ndarray, array,
+                   unsorted_unique, try_to_eval, datestr_to_datetime,
+                   get_numpy_dtype, is_editable_type)
+if ndarray is not FakeObject:
+    from spyderlib.widgets.arrayeditor import ArrayEditor
 from spyderlib.widgets.texteditor import TextEditor
 from spyderlib.widgets.importwizard import ImportWizard
 
-#----Numpy arrays support
-class FakeObject(object):
-    """Fake class used in replacement of missing modules"""
-    pass
-try:
-    from numpy import ndarray
-    from numpy import array, matrix #@UnusedImport (object eval)
-    from spyderlib.widgets.arrayeditor import ArrayEditor
-except ImportError:
-    class ndarray(FakeObject):
-        """Fake ndarray"""
-        pass
 
-#----PIL Images support
-try:
-    from PIL.Image import Image
-    import PIL.Image
-except:
-    class Image(FakeObject):
-        """Fake PIL Image"""
-        pass
-
-#----Misc.
-def address(obj):
-    """Return object address as a string: '<classname @ address>'"""
-    return "<%s @ %s>" % (obj.__class__.__name__,
-                          hex(id(obj)).upper().replace('X','x'))
-
-#----date and datetime objects support
-import datetime
-try:
-    from dateutil.parser import parse as dateparse
-except ImportError:
-    from string import atoi
-    def dateparse(datestr):
-        """Just for 'year, month, day' strings"""
-        return datetime.datetime( *map(atoi, datestr.split(',')) )
-def datestr_to_datetime(value):
-    rp = value.rfind('(')+1
-    return dateparse(value[rp:-1])
-
-#----Background colors for supported types 
-COLORS = {
-          bool:               "#ff00ff",
-          (int, float, long): "#0000ff",
-          list:               "#ffff00",
-          dict:               "#00ffff",
-          tuple:              "#c0c0c0",
-          (str, unicode):     "#800000",
-          ndarray:            "#00ff00",
-          Image:              "#008000",
-          datetime.date:      "#808000",
-          }
-
-def get_color_name(value):
-    """Return color name depending on value type"""
-    if not is_known_type(value):
-        return "#7755aa"
-    for typ, name in COLORS.iteritems():
-        if isinstance(value, typ):
-            return name
-    else:
-        return "#ffffff"
-
-#----Sorting
-def sort_against(lista, listb, reverse=False):
-    """Arrange lista items in the same order as sorted(listb)"""
-    return [item for _, item in sorted(zip(listb, lista), reverse=reverse)]
-
-def unsorted_unique(lista):
-    """Removes duplicates from lista neglecting its initial ordering"""
-    set = {}
-    map(set.__setitem__,lista,[])
-    return set.keys()
-
-#----Display <--> Value
-def value_to_display(value, truncate=False,
-                     trunc_len=80, minmax=False, collvalue=True):
-    """Convert value for display purpose"""
-    if minmax and isinstance(value, ndarray):
-        if value.size == 0:
-            return repr(value)
-        try:
-            return 'Min: %r\nMax: %r' % (value.min(), value.max())
-        except TypeError:
-            pass
-    if isinstance(value, Image):
-        return '%s  Mode: %s' % (address(value), value.mode)
-    if not isinstance(value, (str, unicode)):
-        if isinstance(value, (list, tuple, dict, set)) and not collvalue:            
-            value = address(value)
-        else:
-            value = repr(value)
-    if truncate and len(value) > trunc_len:
-        value = value[:trunc_len].rstrip() + ' ...'
-    return value
-
-def try_to_eval(value):
-    """Try to eval value"""
-    try:
-        return eval(value)
-    except (NameError, SyntaxError, ImportError):
-        return value
-    
 def display_to_value(value, default_value, ignore_errors=True):
     """Convert back to value"""
-    value = unicode(value.toString())
+    value = from_qvariant(value, unicode)
     try:
-        if isinstance(default_value, str):
+        np_dtype = get_numpy_dtype(default_value)
+        if isinstance(default_value, bool):
+            # We must test for boolean before NumPy data types
+            # because `bool` class derives from `int` class
+            try:
+                value = bool(float(value))
+            except ValueError:
+                value = value.lower() == "true"
+        elif np_dtype is not None:
+            if 'complex' in str(type(default_value)):
+                value = np_dtype(complex(value))
+            else:
+                value = np_dtype(value)
+        elif isinstance(default_value, str):
             value = str(value)
         elif isinstance(default_value, unicode):
             value = unicode(value)
+        elif isinstance(default_value, complex):
+            value = complex(value)
         elif isinstance(default_value, float):
             value = float(value)
         elif isinstance(default_value, int):
@@ -166,39 +87,21 @@ def display_to_value(value, default_value, ignore_errors=True):
             raise
     return value
 
-def get_size(item):
-    """Return size of an item of arbitrary type"""
-    if isinstance(item, (list, tuple, dict)):
-        return len(item)
-    elif isinstance(item, ndarray):
-        return item.shape
-    elif isinstance(item, Image):
-        return item.size
-    else:
-        return 1
-
-def get_type_string(item):
-    """Return type string of an object"""
-    if isinstance(item, ndarray):
-        return item.dtype.name
-    elif isinstance(item, Image):
-        return "Image"
-    else:
-        found = re.findall(r"<type '([\S]*)'>", str(type(item)))
-        if found:
-            return found[0]
-
-def get_type(item):
-    """Return type of an item"""
-    text = get_type_string(item)
-    if text is None:
-        text = unicode(_('unknown'))
-    return text[text.find('.')+1:]
-
-def is_known_type(item):
-    """Return True if object has a known type"""
-    return get_type_string(item) is not None
 
+class ProxyObject(object):
+    """Dictionary proxy to an unknown object"""
+    def __init__(self, obj):
+        self.__obj__ = obj
+    
+    def __len__(self):
+        return len(dir(self.__obj__))
+    
+    def __getitem__(self, key):
+        return getattr(self.__obj__, key)
+    
+    def __setitem__(self, key, value):
+        setattr(self.__obj__, key, value)
+        
 
 class ReadOnlyDictModel(QAbstractTableModel):
     """DictEditor Read-Only Table Model"""
@@ -230,7 +133,8 @@ class ReadOnlyDictModel(QAbstractTableModel):
     def set_data(self, data, dictfilter=None):
         """Set model data"""
         self._data = data
-        if dictfilter is not None and not self.remote:
+        if dictfilter is not None and not self.remote and\
+           isinstance(data, (tuple, list, dict)):
             data = dictfilter(data)
         self.showndata = data
         self.header0 = _("Index")
@@ -248,7 +152,11 @@ class ReadOnlyDictModel(QAbstractTableModel):
             if not self.names:
                 self.header0 = _("Key")
         else:
-            raise TypeError("Invalid data type")
+            self.keys = dir(data)
+            self._data = data = self.showndata = ProxyObject(data)
+            self.title += _("Object")
+            if not self.names:
+                self.header0 = _("Attribute")
         self.title += ' ('+str(len(self.keys))+' '+ _("elements")+')'
         if self.remote:
             self.sizes = [ data[self.keys[index]]['size']
@@ -258,7 +166,7 @@ class ReadOnlyDictModel(QAbstractTableModel):
         else:
             self.sizes = [ get_size(data[self.keys[index]])
                            for index in range(len(self.keys)) ]
-            self.types = [ get_type(data[self.keys[index]])
+            self.types = [ get_human_readable_type(data[self.keys[index]])
                            for index in range(len(self.keys)) ]
         self.reset()
 
@@ -333,7 +241,7 @@ class ReadOnlyDictModel(QAbstractTableModel):
     def data(self, index, role=Qt.DisplayRole):
         """Cell content"""
         if not index.isValid():
-            return QVariant()
+            return to_qvariant()
         value = self.get_value(index)
         if index.column() == 3 and self.remote:
             value = value['view']
@@ -342,39 +250,39 @@ class ReadOnlyDictModel(QAbstractTableModel):
                                minmax=self.minmax,
                                collvalue=self.collvalue or index.column() != 3)
         if role == Qt.DisplayRole:
-            return QVariant(display)
+            return to_qvariant(display)
         elif role == Qt.EditRole:
-            return QVariant(value_to_display(value))
+            return to_qvariant(value_to_display(value))
         elif role == Qt.TextAlignmentRole:
             if index.column() == 3:
                 if len(display.splitlines()) < 3:
-                    return QVariant(int(Qt.AlignLeft|Qt.AlignVCenter))
+                    return to_qvariant(int(Qt.AlignLeft|Qt.AlignVCenter))
                 else:
-                    return QVariant(int(Qt.AlignLeft|Qt.AlignTop))
+                    return to_qvariant(int(Qt.AlignLeft|Qt.AlignTop))
             else:
-                return QVariant(int(Qt.AlignLeft|Qt.AlignVCenter))
+                return to_qvariant(int(Qt.AlignLeft|Qt.AlignVCenter))
         elif role == Qt.BackgroundColorRole:
-            return QVariant( self.get_bgcolor(index) )
+            return to_qvariant( self.get_bgcolor(index) )
         elif role == Qt.FontRole:
             if index.column() < 3:
-                return QVariant(get_font('dicteditor_header'))
+                return to_qvariant(get_font('dicteditor_header'))
             else:
-                return QVariant(get_font('dicteditor'))
-        return QVariant()
+                return to_qvariant(get_font('dicteditor'))
+        return to_qvariant()
     
     def headerData(self, section, orientation, role=Qt.DisplayRole):
         """Overriding method headerData"""
         if role != Qt.DisplayRole:
             if role == Qt.FontRole:
-                return QVariant(get_font('dicteditor_header'))
+                return to_qvariant(get_font('dicteditor_header'))
             else:
-                return QVariant()
+                return to_qvariant()
         i_column = int(section)
         if orientation == Qt.Horizontal:
             headers = (self.header0, _("Type"), _("Size"), _("Value"))
-            return QVariant( headers[i_column] )
+            return to_qvariant( headers[i_column] )
         else:
-            return QVariant()
+            return to_qvariant()
 
     def flags(self, index):
         """Overriding method flags"""
@@ -393,12 +301,12 @@ class DictModel(ReadOnlyDictModel):
         self._data[ self.keys[index.row()] ] = value
         self.showndata[ self.keys[index.row()] ] = value
         self.sizes[index.row()] = get_size(value)
-        self.types[index.row()] = get_type(value)
+        self.types[index.row()] = get_human_readable_type(value)
 
     def get_bgcolor(self, index):
         """Background color depending on value"""
         value = self.get_value(index)
-        if index.column()<3:
+        if index.column() < 3:
             color = ReadOnlyDictModel.get_bgcolor(self, index)
         else:
             if self.remote:
@@ -413,7 +321,7 @@ class DictModel(ReadOnlyDictModel):
         """Cell content change"""
         if not index.isValid():
             return False
-        if index.column()<3:
+        if index.column() < 3:
             return False
         value = display_to_value(value, self.get_value(index),
                                  ignore_errors=True)
@@ -440,7 +348,7 @@ class DictDelegate(QItemDelegate):
 
     def createEditor(self, parent, option, index):
         """Overriding method createEditor"""
-        if index.column()<3:
+        if index.column() < 3:
             return None
         try:
             value = self.get_value(index)
@@ -481,7 +389,7 @@ class DictDelegate(QItemDelegate):
             editor = ArrayEditor(parent)
             if not editor.setup_and_check(arr, title=key, readonly=readonly):
                 return
-            conv_func = lambda arr: PIL.Image.fromarray(arr, mode=value.mode)
+            conv_func = lambda arr: Image.fromarray(arr, mode=value.mode)
             self.create_dialog(editor, dict(model=index.model(), editor=editor,
                                             key=key, readonly=readonly,
                                             conv=conv_func))
@@ -509,13 +417,21 @@ class DictDelegate(QItemDelegate):
                                             key=key, readonly=readonly))
             return None
         #---editor = QLineEdit
-        else:
+        elif self.inplace or is_editable_type(value):
             editor = QLineEdit(parent)
             editor.setFont(get_font('dicteditor'))
             editor.setAlignment(Qt.AlignLeft)
             self.connect(editor, SIGNAL("returnPressed()"),
                          self.commitAndCloseEditor)
             return editor
+        #---editor = DictEditor for an arbitrary object
+        else:
+            editor = DictEditor()
+            editor.setup(value, key, icon=self.parent().windowIcon(),
+                         readonly=readonly)
+            self.create_dialog(editor, dict(model=index.model(), editor=editor,
+                                            key=key, readonly=readonly))
+            return None
             
     def create_dialog(self, editor, data):
         self._editors[id(editor)] = data
@@ -565,6 +481,17 @@ class DictDelegate(QItemDelegate):
         
         if isinstance(editor, QLineEdit):
             value = editor.text()
+            try:
+                value = display_to_value(to_qvariant(value),
+                                         self.get_value(index),
+                                         ignore_errors=False)
+            except Exception, msg:
+                raise
+                QMessageBox.critical(editor, _("Edit item"),
+                                     _("<b>Unable to assign data to item.</b>"
+                                       "<br><br>Error message:<br>%s"
+                                       ) % str(msg))
+                return
         elif isinstance(editor, QDateEdit):
             qdate = editor.date()
             value = datetime.date( qdate.year(), qdate.month(), qdate.day() )
@@ -578,20 +505,12 @@ class DictDelegate(QItemDelegate):
         else:
             # Should not happen...
             raise RuntimeError("Unsupported editor widget")
-        
-        try:
-            value = display_to_value(QVariant(value), self.get_value(index),
-                                     ignore_errors=False)
-        except Exception, msg:
-            QMessageBox.critical(editor, _("Edit item"),
-                                 _("<b>Unable to assign data to item.</b>"
-                                   "<br><br>Error message:<br>%s"
-                                   ) % str(msg))
-            return
         self.set_value(index, value)
 
 
 class BaseTableView(QTableView):
+    """Base dictionnary editor table view"""
+    sig_option_changed = Signal(str, object)
     def __init__(self, parent):
         QTableView.__init__(self, parent)
         self.array_filename = None
@@ -630,6 +549,8 @@ class BaseTableView(QTableView):
             self.collvalue_action.setChecked(collvalue)
             return
         
+        resize_action = create_action(self, _("Resize rows to contents"),
+                                      triggered=self.resizeRowsToContents)
         self.paste_action = create_action(self, _("Paste"),
                                           icon=get_icon('editpaste.png'),
                                           triggered=self.paste)
@@ -688,13 +609,15 @@ class BaseTableView(QTableView):
                         self.save_array_action, self.insert_action,
                         self.remove_action, self.copy_action, self.paste_action,
                         None, self.rename_action,self.duplicate_action,
-                        None, self.truncate_action, self.inplace_action,
-                        self.collvalue_action]
+                        None, resize_action, None, self.truncate_action,
+                        self.inplace_action, self.collvalue_action]
         if ndarray is not FakeObject:
             menu_actions.append(self.minmax_action)
         add_actions(menu, menu_actions)
         self.empty_ws_menu = QMenu(self)
-        add_actions(self.empty_ws_menu, [self.insert_action, self.paste_action])
+        add_actions(self.empty_ws_menu,
+                    [self.insert_action, self.paste_action,
+                     None, resize_action])
         return menu
     
     #------ Remote/local API ---------------------------------------------------
@@ -816,16 +739,12 @@ class BaseTableView(QTableView):
         """Reimplement Qt methods"""
         if event.key() == Qt.Key_Delete:
             self.remove_item()
-            event.accept()
         elif event.key() == Qt.Key_F2:
             self.rename_item()
-            event.accept()
         elif event == QKeySequence.Copy:
             self.copy()
-            event.accept()
         elif event == QKeySequence.Paste:
             self.paste()
-            event.accept()
         else:
             QTableView.keyPressEvent(self, event)
         
@@ -841,22 +760,22 @@ class BaseTableView(QTableView):
 
     def toggle_inplace(self, state):
         """Toggle in-place editor option"""
-        self.emit(SIGNAL('option_changed'), 'inplace', state)
+        self.sig_option_changed.emit('inplace', state)
         self.delegate.inplace = state
         
     def toggle_truncate(self, state):
         """Toggle display truncating option"""
-        self.emit(SIGNAL('option_changed'), 'truncate', state)
+        self.sig_option_changed.emit('truncate', state)
         self.model.truncate = state
         
     def toggle_minmax(self, state):
         """Toggle min/max display for numpy arrays"""
-        self.emit(SIGNAL('option_changed'), 'minmax', state)
+        self.sig_option_changed.emit('minmax', state)
         self.model.minmax = state
         
     def toggle_collvalue(self, state):
         """Toggle value display for collections"""
-        self.emit(SIGNAL('option_changed'), 'collvalue', state)
+        self.sig_option_changed.emit('collvalue', state)
         self.model.collvalue = state
             
     def edit_item(self):
@@ -895,7 +814,7 @@ class BaseTableView(QTableView):
         orig_key = self.model.keys[idx_rows[0]]
         new_key, valid = QInputDialog.getText(self, _( 'Rename'), _( 'Key:'),
                                               QLineEdit.Normal,orig_key)
-        if valid and not new_key.isEmpty():
+        if valid and unicode(new_key):
             new_key = try_to_eval(unicode(new_key))
             if new_key == orig_key:
                 return
@@ -925,7 +844,7 @@ class BaseTableView(QTableView):
         elif isinstance(data, dict):
             key, valid = QInputDialog.getText(self, _( 'Insert'), _( 'Key:'),
                                               QLineEdit.Normal)
-            if valid and not key.isEmpty():
+            if valid and unicode(key):
                 key = try_to_eval(unicode(key))
             else:
                 return
@@ -933,7 +852,7 @@ class BaseTableView(QTableView):
             return
         value, valid = QInputDialog.getText(self, _('Insert'), _('Value:'),
                                             QLineEdit.Normal)
-        if valid and not value.isEmpty():
+        if valid and unicode(value):
             self.new_value(key, try_to_eval(unicode(value)))
             
     def __prepare_plot(self):
@@ -943,9 +862,8 @@ class BaseTableView(QTableView):
         except ImportError:
             try:
                 if 'matplotlib' not in sys.modules:
-                    from spyderlib import mpl_patch
-                    mpl_patch.set_backend("Qt4Agg")
-                    mpl_patch.apply()
+                    import matplotlib
+                    matplotlib.use("Qt4Agg")
                 return True
             except ImportError:
                 QMessageBox.warning(self, _("Import error"),
@@ -987,11 +905,12 @@ class BaseTableView(QTableView):
         if self.array_filename is None:
             self.array_filename = os.getcwdu()
         self.emit(SIGNAL('redirect_stdio(bool)'), False)
-        filename = QFileDialog.getSaveFileName(self, title, self.array_filename,
-                          _("NumPy arrays")+" (*.npy)")
+        filename, _selfilter = getsavefilename(self, title,
+                                               self.array_filename,
+                                               _("NumPy arrays")+" (*.npy)")
         self.emit(SIGNAL('redirect_stdio(bool)'), True)
         if filename:
-            self.array_filename = unicode(filename)
+            self.array_filename = filename
             data = self.delegate.get_value( self.currentIndex() )
             try:
                 import numpy as np
@@ -1178,6 +1097,13 @@ class DictEditor(QDialog):
     """Dictionary/List Editor Dialog"""
     def __init__(self, parent=None):
         QDialog.__init__(self, parent)
+        
+        # Destroying the C++ object right after closing the dialog box,
+        # otherwise it may be garbage-collected in another QThread
+        # (e.g. the editor's analysis thread in Spyder), thus leading to
+        # a segmentation fault on UNIX or an application crash on Windows
+        self.setAttribute(Qt.WA_DeleteOnClose)
+        
         self.data_copy = None
         self.widget = None
         
@@ -1186,9 +1112,16 @@ class DictEditor(QDialog):
         if isinstance(data, dict):
             # dictionnary
             self.data_copy = data.copy()
-        else:
+            datalen = len(data)
+        elif isinstance(data, (tuple, list)):
             # list, tuple
             self.data_copy = data[:]
+            datalen = len(data)
+        else:
+            # unknown object
+            import copy
+            self.data_copy = copy.deepcopy(data)
+            datalen = len(dir(data))
         self.widget = DictEditorWidget(self, self.data_copy, title=title,
                                        readonly=readonly, remote=remote)
         
@@ -1209,7 +1142,7 @@ class DictEditor(QDialog):
         constant = 121
         row_height = 30
         error_margin = 20
-        height = constant + row_height*min([20, len(data)]) + error_margin
+        height = constant + row_height*min([20, datalen]) + error_margin
         self.resize(width, height)
         
         self.setWindowTitle(self.widget.get_title())
@@ -1221,26 +1154,9 @@ class DictEditor(QDialog):
         
     def get_value(self):
         """Return modified copy of dictionary or list"""
+        # It is import to avoid accessing Qt C++ object as it has probably
+        # already been destroyed, due to the Qt.WA_DeleteOnClose attribute
         return self.data_copy
-    
-    
-def dedit(seq):
-    """
-    Edit the sequence 'seq' in a GUI-based editor and return the edited copy
-    (if Cancel is pressed, return None)
-
-    The object 'seq' is a container (dict, list or tuple)
-
-    (instantiate a new QApplication if necessary,
-    so it can be called directly from the interpreter)
-    """
-    app = qapplication()
-    dialog = DictEditor()
-    dialog.setup(seq)
-    dialog.show()
-    app.exec_()
-    if dialog.result():
-        return dialog.get_value()
 
 
 #----Remote versions of DictDelegate and DictEditorTableView
@@ -1331,7 +1247,7 @@ class RemoteDictEditorTableView(BaseTableView):
             
     def toggle_remote_editing(self, state):
         """Toggle remote editing state"""
-        self.emit(SIGNAL('option_changed'), 'remote_editing', state)
+        self.sig_option_changed.emit('remote_editing', state)
         self.remote_editing_enabled = state
             
     def edit_item(self):
@@ -1358,59 +1274,40 @@ class RemoteDictEditorTableView(BaseTableView):
             BaseTableView.edit_item(self)
 
 
-#----Globals filter: filter namespace dictionaries (to be edited in DictEditor)
-def is_supported(value, iter=0, itermax=-1, filters=None):
-    """Return True if the value is supported, False otherwise"""
-    assert filters is not None
-    if iter == itermax:
-        return True
-    elif not isinstance(value, filters):
-        return False
-    elif isinstance(value, (list, tuple, set)):
-        for val in value:
-            if not is_supported(val, iter+1, filters=filters):
-                return False
-    elif isinstance(value, dict):
-        for key, val in value.iteritems():
-            if not is_supported(key, iter+1, filters=filters) \
-               or not is_supported(val, iter+1, filters=filters):
-                return False
-    return True
-
-def globalsfilter(input_dict, itermax=-1, filters=None,
-                  exclude_private=None, exclude_upper=None,
-                  exclude_unsupported=None, excluded_names=None):
-    """Keep only objects that can be pickled"""
-    output_dict = {}
-    for key, value in input_dict.items():
-        excluded = (exclude_private and key.startswith('_')) or \
-                   (exclude_upper and key[0].isupper()) or \
-                   (key in excluded_names) or \
-                   (exclude_unsupported and \
-                    not is_supported(value, itermax=itermax, filters=filters))
-        if not excluded:
-            output_dict[key] = value
-    return output_dict
-
-
 def get_test_data():
     """Create test data"""
     import numpy as np
-    image = PIL.Image.fromarray(np.random.random_integers(255, size=(100, 100)))
+    from spyderlib.pil_patch import Image
+    image = Image.fromarray(np.random.random_integers(255, size=(100, 100)))
     testdict = {'d': 1, 'a': np.random.rand(10, 10), 'b': [1, 2]}
     testdate = datetime.date(1945, 5, 8)
-    return {'str': 'kjkj kj k j j kj k jkj',
+    class Foobar(object):
+        def __init__(self):
+            self.text = "toto"
+            self.testdict = testdict
+            self.testdate = testdate
+    foobar = Foobar()
+    return {'object': foobar,
+            'str': 'kjkj kj k j j kj k jkj',
             'unicode': u'éù',
             'list': [1, 3, [sorted, 5, 6], 'kjkj', None],
             'tuple': ([1, testdate, testdict], 'kjkj', None),
             'dict': testdict,
             'float': 1.2233,
+            'int': 223,
+            'bool': True,
             'array': np.random.rand(10, 10),
             '1D-array': np.linspace(-10, 10),
             'empty_array': np.array([]),
             'image': image,
             'date': testdate,
             'datetime': datetime.datetime(1945, 5, 8),
+            'complex': 2+1j,
+            'complex64': np.complex64(2+1j),
+            'int8_scalar': np.int8(8),
+            'int16_scalar': np.int16(16),
+            'int32_scalar': np.int32(32),
+            'bool_scalar': np.bool(8),
             'unsupported1': np.arccos,
             'unsupported2': np.cast,
             1: (1, 2, 3), -5: ("a", "b", "c"), 2.5: np.array((4.0, 6.0, 8.0)),            
@@ -1418,8 +1315,11 @@ def get_test_data():
 
 def test():
     """Dictionary editor test"""
-    out = dedit( get_test_data() )
-    print "out:", out
+    _app = qapplication() #analysis:ignore
+    dialog = DictEditor()
+    dialog.setup(get_test_data())
+    if dialog.exec_():
+        print "out:", dialog.get_value()
     
 def remote_editor_test():
     """Remote dictionary editor test"""
