@@ -6,71 +6,26 @@
 
 """Editor tools: outline explorer, etc."""
 
-import sys, re,  os.path as osp
+import sys
+import re
+import os.path as osp
 
 from spyderlib.qt.QtGui import (QWidget, QTreeWidgetItem,  QHBoxLayout,
                                 QVBoxLayout)
 from spyderlib.qt.QtCore import Qt, SIGNAL
+from spyderlib.qt.compat import from_qvariant
 
 # For debugging purpose:
 STDOUT = sys.stdout
 
 # Local import
-from spyderlib.config import get_icon, _
+from spyderlib.baseconfig import _
+from spyderlib.config import get_icon
 from spyderlib.utils.qthelpers import (create_action, create_toolbutton,
                                        set_item_user_text)
 from spyderlib.widgets.onecolumntree import OneColumnTree
 
 
-#===============================================================================
-# Pyflakes code analysis
-#===============================================================================
-import compiler
-
-def check(source_code, filename=None):
-    try:
-        import pyflakes.checker, pyflakes.messages
-    except ImportError:
-        return []
-    try:
-#        tree = compiler.parse(file(filename, 'U').read() + '\n')
-        tree = compiler.parse(source_code+'\n')
-    except (SyntaxError, IndentationError), e:
-        message = e.args[0]
-        value = sys.exc_info()[1]
-        try:
-            (lineno, _offset, _text) = value[1][1:]
-        except IndexError:
-            # Could not compile script
-            return
-        return [ (message, lineno, True) ]
-    else:
-        results = []
-        if filename is None:
-            filename = '(none)'
-        w = pyflakes.checker.Checker(tree, filename)
-        w.messages.sort(lambda a, b: cmp(a.lineno, b.lineno))
-        
-        error_messages = (pyflakes.messages.UndefinedName,
-                          pyflakes.messages.UndefinedExport,
-                          pyflakes.messages.UndefinedLocal,
-                          pyflakes.messages.DuplicateArgument,
-                          pyflakes.messages.LateFutureImport)
-        lines = source_code.splitlines()
-        for warning in w.messages:
-            if 'pyflakes:ignore' not in lines[warning.lineno-1]:
-                results.append( (warning.message % warning.message_args,
-                                 warning.lineno,
-                                 isinstance(warning, error_messages)) )
-        return results
-
-if __name__ == '__main__':
-    fname = __file__
-    check_results = check(file(fname, 'U').read(), fname)
-    for message, line, error in check_results:
-        print "Message: %s -- Line: %s -- Error? %s" % (message, line, error)
-
-
 #===============================================================================
 # Class browser
 #===============================================================================
@@ -133,7 +88,7 @@ class TreeItem(QTreeWidgetItem):
                 QTreeWidgetItem.__init__(self, parent, preceding,
                                          QTreeWidgetItem.Type)
         self.setText(0, name)
-        parent_text = unicode(parent.data(0, Qt.UserRole).toString())
+        parent_text = from_qvariant(parent.data(0, Qt.UserRole), unicode)
         set_item_user_text(self, parent_text+'/'+name)
         self.line = line
         
@@ -511,7 +466,7 @@ class OutlineExplorerTreeWidget(OneColumnTree):
                 break
     
     
-class OutlineExplorer(QWidget):
+class OutlineExplorerWidget(QWidget):
     """
     Class browser
     
