@@ -19,22 +19,29 @@ STDERR = sys.stderr
 from spyderlib.qt.QtGui import (QApplication, QWidget, QVBoxLayout,
                                 QHBoxLayout, QMenu, QLabel, QInputDialog,
                                 QLineEdit, QToolButton)
-from spyderlib.qt.QtCore import (QProcess, SIGNAL, QByteArray, QString, QTimer,
-                                 QStringList, Qt)
+from spyderlib.qt.QtCore import (QProcess, SIGNAL, QByteArray, QTimer, Qt,
+                                 QTextCodec)
+locale_codec = QTextCodec.codecForLocale()
 
 # Local imports
 from spyderlib.utils.qthelpers import (create_toolbutton, create_action,
                                        add_actions)
-from spyderlib.config import get_icon, get_conf_path, _
+from spyderlib.baseconfig import get_conf_path, _
+from spyderlib.config import get_icon
 
 
 def add_pathlist_to_PYTHONPATH(env, pathlist):
-    assert isinstance(env, QStringList)
+    # PyQt API 1/2 compatibility-related tests:
+    assert isinstance(env, list)
+    assert all([isinstance(path, basestring) for path in env])
+    
     pypath = "PYTHONPATH"
     pathstr = os.pathsep.join(pathlist)
     if os.environ.get(pypath) is not None:
-        env.replaceInStrings(pypath+'=', pypath+'='+pathstr+os.pathsep,
-                             Qt.CaseSensitive)
+        for index, var in enumerate(env[:]):
+            if var.startswith(pypath+'='):
+                env[index] = var.replace(pypath+'=',
+                                         pypath+'='+pathstr+os.pathsep)
         env.append('OLD_PYTHONPATH='+os.environ[pypath])
     else:
         env.append(pypath+'='+pathstr)
@@ -195,6 +202,7 @@ class ExternalShellBase(QWidget):
         if self.process is not None:
             self.is_closing = True
             self.process.kill()
+            self.process.waitForFinished(100)
         self.disconnect(self.timer, SIGNAL("timeout()"), self.show_time)
     
     def set_running_state(self, state=True):
@@ -247,7 +255,7 @@ class ExternalShellBase(QWidget):
 #    Input/Output
 #===============================================================================
     def transcode(self, bytes):
-        return unicode( QString.fromLocal8Bit(bytes.data()) )
+        return unicode( locale_codec.toUnicode(bytes.data()) )
     
     def get_stdout(self):
         self.process.setReadChannel(QProcess.StandardOutput)
@@ -276,7 +284,7 @@ class ExternalShellBase(QWidget):
         byte_array.append(char)
         self.process.write(byte_array)
         self.process.waitForBytesWritten(-1)
-        self.shell.write(QString(byte_array), flush=True)
+        self.shell.write(locale_codec.toUnicode(byte_array), flush=True)
         
     def keyboard_interrupt(self):
         raise NotImplementedError
@@ -291,7 +299,7 @@ def test():
     from spyderlib.plugins.variableexplorer import VariableExplorer
     settings = VariableExplorer.get_settings()
     shell = ExternalPythonShell(wdir=osp.dirname(spyderlib.__file__),
-                                ipython=True, stand_alone=settings,
+                                ipython_kernel=True, stand_alone=settings,
                                 arguments="-q4thread -pylab -colors LightBG",
                                 mpl_patch_enabled=True, light_background=False)
 #    shell = ExternalPythonShell(wdir=osp.dirname(spyderlib.__file__),
