@@ -13,19 +13,10 @@
 
 from __future__ import with_statement
 
-try:
-    # PyQt4 4.3.3 on Windows (static DLLs) with py2exe installed:
-    # -> pythoncom must be imported first, otherwise py2exe's boot_com_servers
-    #    will raise an exception ("ImportError: DLL load failed [...]") when
-    #    calling any of the QFileDialog static methods (getOpenFileName, ...)
-    import pythoncom #@UnusedImport
-except ImportError:
-    pass
-
 from spyderlib.qt.QtGui import (QHBoxLayout, QWidget, QTreeWidgetItem,
-                                QSizePolicy, QRadioButton, QVBoxLayout, QLabel,
-                                QFileDialog)
+                                QSizePolicy, QRadioButton, QVBoxLayout, QLabel)
 from spyderlib.qt.QtCore import SIGNAL, Qt, QThread, QMutexLocker, QMutex
+from spyderlib.qt.compat import getexistingdirectory
 
 import sys, os, re, fnmatch
 import os.path as osp
@@ -35,10 +26,11 @@ from subprocess import Popen, PIPE
 STDOUT = sys.stdout
 
 # Local imports
-from spyderlib.utils.programs import is_program_installed
+from spyderlib.utils import programs
 from spyderlib.utils.qthelpers import (get_std_icon, create_toolbutton,
                                        get_filetype_icon)
-from spyderlib.config import get_icon, _
+from spyderlib.baseconfig import _
+from spyderlib.config import get_icon
 from spyderlib.widgets.comboboxes import PathComboBox, PatternComboBox
 from spyderlib.widgets.onecolumntree import OneColumnTree
 
@@ -48,6 +40,7 @@ def abspardir(path):
     return osp.abspath(osp.join(path, os.pardir))
 
 def get_common_path(pathlist):
+    """Return common path for all paths in pathlist"""
     common = osp.commonprefix(pathlist)
     if len(common) > 1:
         if not osp.isdir(common):
@@ -55,9 +48,11 @@ def get_common_path(pathlist):
         return osp.abspath(common)
 
 def is_hg_installed():
-    return is_program_installed('hg.exe' if os.name == 'nt' else 'hg')
+    """Return True if Mercurial is installed"""
+    return programs.find_program('hg') is not None
 
 def get_hg_root(path):
+    """Return Mercurial root directory path"""
     previous_path = path
     while not osp.isdir(osp.join(path, '.hg')):
         path = abspardir(path)
@@ -131,6 +126,7 @@ def get_hg_root(path):
 #    return results, nb
 
 class SearchThread(QThread):
+    """Find in files search thread"""
     def __init__(self, parent):
         QThread.__init__(self, parent)
         self.mutex = QMutex()
@@ -296,9 +292,7 @@ class SearchThread(QThread):
 
 
 class FindOptions(QWidget):
-    """
-    Find widget with options
-    """
+    """Find widget with options"""
     def __init__(self, parent, search_text, search_text_regexp, search_path,
                  include, include_idx, include_regexp,
                  exclude, exclude_idx, exclude_regexp,
@@ -421,6 +415,7 @@ class FindOptions(QWidget):
                      lambda valid: self.emit(SIGNAL('find()')))
             
         vlayout = QVBoxLayout()
+        vlayout.setContentsMargins(0, 0, 0, 0)
         vlayout.addLayout(hlayout1)
         vlayout.addLayout(hlayout2)
         vlayout.addLayout(hlayout3)
@@ -458,8 +453,9 @@ class FindOptions(QWidget):
             self.custom_dir.setChecked(True)
         
     def set_search_text(self, text):
-        self.search_text.setEditText(text)
-        self.search_text.lineEdit().selectAll()
+        if text:
+            self.search_text.add_text(text)
+            self.search_text.lineEdit().selectAll()
         self.search_text.setFocus()
         
     def get_options(self, all=False):
@@ -514,10 +510,9 @@ class FindOptions(QWidget):
     def select_directory(self):
         """Select directory"""
         self.parent().emit(SIGNAL('redirect_stdio(bool)'), False)
-        directory = QFileDialog.getExistingDirectory(self,
-                                                 _("Select directory"),
-                                                 self.dir_combo.currentText())
-        if not directory.isEmpty():
+        directory = getexistingdirectory(self, _("Select directory"),
+                                         self.dir_combo.currentText())
+        if directory:
             self.set_directory(directory)
         self.parent().emit(SIGNAL('redirect_stdio(bool)'), True)
         
@@ -536,9 +531,8 @@ class FindOptions(QWidget):
             # Toggle find widgets
             self.parent().emit(SIGNAL('toggle_visibility(bool)'),
                                not self.isVisible())
-            event.accept()
         else:
-            event.ignore()
+            QWidget.keyPressEvent(self, event)
 
 
 class ResultsBrowser(OneColumnTree):
@@ -700,9 +694,8 @@ class FindInFilesWidget(QWidget):
         
         self.setWindowTitle(_('Find in files'))
 
-        self.search_thread = SearchThread(self)
-        self.connect(self.search_thread, SIGNAL("finished(bool)"),
-                     self.search_complete)
+        self.search_thread = None
+        self.get_pythonpath_callback = None
         
         self.find_options = FindOptions(self, search_text, search_text_regexp,
                                         search_path,
@@ -711,7 +704,8 @@ class FindInFilesWidget(QWidget):
                                         supported_encodings, in_python_path,
                                         more_options)
         self.connect(self.find_options, SIGNAL('find()'), self.find)
-        self.connect(self.find_options, SIGNAL('stop()'), self.stop)
+        self.connect(self.find_options, SIGNAL('stop()'),
+                     self.stop_and_reset_thread)
         
         self.result_browser = ResultsBrowser(self)
         
@@ -746,29 +740,48 @@ class FindInFilesWidget(QWidget):
         self.setLayout(layout)
             
     def set_search_text(self, text):
+        """Set search pattern"""
         self.find_options.set_search_text(text)
-        
-    def set_pythonpath_callback(self, callback):
-        self.search_thread.get_pythonpath_callback = callback
 
     def find(self):
         """Call the find function"""
         options = self.find_options.get_options()
         if options is None:
             return
+        self.stop_and_reset_thread(ignore_results=True)
+        self.search_thread = SearchThread(self)
+        self.search_thread.get_pythonpath_callback = \
+                                                self.get_pythonpath_callback
+        self.connect(self.search_thread, SIGNAL("finished(bool)"),
+                     self.search_complete)
         self.search_thread.initialize(*options)
         self.search_thread.start()
         self.find_options.ok_button.setEnabled(False)
         self.find_options.stop_button.setEnabled(True)
             
-    def stop(self):
-        if self.search_thread.isRunning():
-            self.search_thread.stop()
-            
+    def stop_and_reset_thread(self, ignore_results=False):
+        """Stop current search thread and clean-up"""
+        if self.search_thread is not None:
+            if self.search_thread.isRunning():
+                if ignore_results:
+                    self.disconnect(self.search_thread,
+                                    SIGNAL("finished(bool)"),
+                                    self.search_complete)
+                self.search_thread.stop()
+                self.search_thread.wait()
+            self.search_thread.setParent(None)
+            self.search_thread = None
+        
+    def closing_widget(self):
+        """Perform actions before widget is closed"""
+        self.stop_and_reset_thread(ignore_results=True)
+        
     def search_complete(self, completed):
+        """Current search thread has finished"""
         self.find_options.ok_button.setEnabled(True)
         self.find_options.stop_button.setEnabled(False)
         found = self.search_thread.get_results()
+        self.stop_and_reset_thread()
         if found is not None:
             results, pathlist, nb, error_flag = found
             search_text = unicode( self.find_options.search_text.currentText() )
