@@ -34,11 +34,12 @@ OTHER DEALINGS IN THE SOFTWARE.
 """
 
 # History:
+# 1.0.11: added support for PySide
 # 1.0.10: added float validator (disable "Ok" and "Apply" button when not valid)
 # 1.0.7: added support for "Apply" button
 # 1.0.6: code cleaning
 
-__version__ = '1.0.10'
+__version__ = '1.0.11'
 __license__ = __doc__
 
 DEBUG = False
@@ -58,12 +59,11 @@ from spyderlib.qt.QtGui import (QWidget, QLineEdit, QComboBox, QLabel,
                                 QTabWidget, QApplication, QStackedWidget,
                                 QDateEdit, QDateTimeEdit, QFont, QFontComboBox,
                                 QFontDatabase, QGridLayout, QDoubleValidator)
-from spyderlib.qt.QtCore import (Qt, SIGNAL, SLOT, QSize, QString,
-                                 pyqtSignature, pyqtProperty)
+from spyderlib.qt.QtCore import Qt, SIGNAL, SLOT, QSize, Slot, Property
 import datetime
 
 # Local imports
-from spyderlib.config import _
+from spyderlib.baseconfig import _
 
 
 class ColorButton(QPushButton):
@@ -89,7 +89,7 @@ class ColorButton(QPushButton):
     def get_color(self):
         return self._color
     
-    @pyqtSignature("QColor")
+    @Slot(QColor)
     def set_color(self, color):
         if color != self._color:
             self._color = color
@@ -98,7 +98,7 @@ class ColorButton(QPushButton):
             pixmap.fill(color)
             self.setIcon(QIcon(pixmap))
     
-    color = pyqtProperty("QColor", get_color, set_color)
+    color = Property("QColor", get_color, set_color)
 
 
 def text_to_qcolor(text):
@@ -107,7 +107,7 @@ def text_to_qcolor(text):
     Avoid warning from Qt when an invalid QColor is instantiated
     """
     color = QColor()
-    if isinstance(text, QString):
+    if not isinstance(text, basestring): # testing for QString (PyQt API#1)
         text = str(text)
     if not isinstance(text, (unicode, str)):
         return color
@@ -268,6 +268,7 @@ class FormWidget(QWidget):
             elif isinstance(value, (str, unicode)):
                 field = QLineEdit(value, self)
             elif isinstance(value, (list, tuple)):
+                value = list(value)  # in case this is a tuple
                 selindex = value.pop(0)
                 field = QComboBox(self)
                 if isinstance(value[0], (list, tuple)):
@@ -323,10 +324,13 @@ class FormWidget(QWidget):
                 value = unicode(field.text())
             elif isinstance(value, (list, tuple)):
                 index = int(field.currentIndex())
-                if isinstance(value[0], (list, tuple)):
-                    value = value[index][0]
+                if isinstance(value[0], int):
+                    # Return an int index, if initialization was an int
+                    value = index
                 else:
-                    value = value[index]
+                    value = value[index+1]
+                    if isinstance(value, (list, tuple)):
+                        value = value[0]
             elif isinstance(value, bool):
                 value = field.checkState() == Qt.Checked
             elif isinstance(value, float):
@@ -465,6 +469,8 @@ class FormDialog(QDialog):
         
     def get(self):
         """Return form result"""
+        # It is import to avoid accessing Qt C++ object as it has probably
+        # already been destroyed, due to the Qt.WA_DeleteOnClose attribute
         return self.data
 
 
@@ -496,7 +502,6 @@ def fedit(data, title="", comment="", icon=None, parent=None, apply=None):
           * the first element will be the selected index (or value)
           * the other elements can be couples (key, value) or only values
     """
-    
     # Create a QApplication instance if no instance currently exists
     # (e.g. if the module is used directly from the interpreter)
     if QApplication.startingUp():
@@ -507,7 +512,6 @@ def fedit(data, title="", comment="", icon=None, parent=None, apply=None):
         return dialog.get()
 
 
-
 if __name__ == "__main__":
 
     def create_datalist_example():
