@@ -54,8 +54,9 @@ def oedit(obj, modal=True, namespace=None):
     """
     # Local import
     from spyderlib.widgets.texteditor import TextEditor
-    from spyderlib.widgets.dicteditor import (DictEditor, ndarray, FakeObject,
-                                              Image, is_known_type)
+    from spyderlib.widgets.dicteditorutils import (ndarray, FakeObject,
+                                                   Image, is_known_type)
+    from spyderlib.widgets.dicteditor import DictEditor
     from spyderlib.widgets.arrayeditor import ArrayEditor
     
     from spyderlib.utils.qthelpers import qapplication
@@ -70,6 +71,8 @@ def oedit(obj, modal=True, namespace=None):
             namespace = globals()
         keeper.set_namespace(namespace)
         obj = namespace[obj_name]
+        # keep QApplication reference alive in the Python interpreter:
+        namespace['__qapp__'] = app
 
     conv_func = lambda data: data
     readonly = not is_known_type(obj)
@@ -86,35 +89,35 @@ def oedit(obj, modal=True, namespace=None):
         if not dialog.setup_and_check(data, title=obj_name,
                                       readonly=readonly):
             return
-        import PIL.Image
-        conv_func = lambda data: PIL.Image.fromarray(data, mode=obj.mode)
+        from spyderlib.pil_patch import Image
+        conv_func = lambda data: Image.fromarray(data, mode=obj.mode)
     elif isinstance(obj, (str, unicode)):
         dialog = TextEditor(obj, title=obj_name, readonly=readonly)
-    elif isinstance(obj, (dict, tuple, list)):
+    else:
         dialog = DictEditor()
         dialog.setup(obj, title=obj_name, readonly=readonly)
-    else:
-        raise RuntimeError("Unsupported datatype")
     
     def end_func(dialog):
         return conv_func(dialog.get_value())
     
     if modal:
-        dialog.exec_()
-        if dialog.result():
+        if dialog.exec_():
             return end_func(dialog)
     else:
         keeper.create_dialog(dialog, obj_name, end_func)
         import os
-        if os.environ.get("REMOVE_PYQT_INPUTHOOK", "").lower() == "true" \
+        replaced_pyqt_inputhook = os.environ.get("REPLACE_PYQT_INPUTHOOK", ""
+                                                 ).lower() == "true"
+        if os.name == 'nt' and not replaced_pyqt_inputhook \
            and not os.environ.get('IPYTHON', False):
             app.exec_()
 
 
 def test():
     """Run object editor test"""
-    import datetime, numpy as np, PIL.Image
-    image = PIL.Image.fromarray(np.random.random_integers(255, size=(100, 100)))
+    import datetime, numpy as np
+    from spyderlib.pil_patch import Image
+    image = Image.fromarray(np.random.random_integers(255, size=(100, 100)))
     example = {'str': 'kjkj kj k j j kj k jkj',
                'list': [1, 3, 4, 'kjkj', None],
                'dict': {'d': 1, 'a': np.random.rand(10, 10), 'b': [1, 2]},
@@ -125,6 +128,11 @@ def test():
                'datetime': datetime.datetime(1945, 5, 8),
                }
     image = oedit(image)
+    class Foobar(object):
+        def __init__(self):
+            self.text = "toto"
+    foobar = Foobar()
+    print oedit(foobar)
     print oedit(example)
     print oedit(np.random.rand(10, 10))
     print oedit(oedit.__doc__)
