@@ -8,10 +8,11 @@
 
 from __future__ import with_statement
 
-from spyderlib.qt.QtGui import (QDialog, QListWidget, QFileDialog,
-                                QDialogButtonBox, QVBoxLayout, QHBoxLayout,
-                                QMessageBox, QListWidgetItem)
+from spyderlib.qt.QtGui import (QDialog, QListWidget, QDialogButtonBox,
+                                QVBoxLayout, QHBoxLayout, QMessageBox,
+                                QListWidgetItem)
 from spyderlib.qt.QtCore import Qt, SIGNAL, SLOT
+from spyderlib.qt.compat import getexistingdirectory
 
 import os, sys, os.path as osp
 
@@ -20,13 +21,20 @@ STDOUT = sys.stdout
 
 # Local imports
 from spyderlib.utils.qthelpers import get_std_icon, create_toolbutton
-from spyderlib.config import get_icon, _
+from spyderlib.baseconfig import _
+from spyderlib.config import get_icon
 
 
 class PathManager(QDialog):
     def __init__(self, parent=None, pathlist=None, ro_pathlist=None, sync=True):
         QDialog.__init__(self, parent)
         
+        # Destroying the C++ object right after closing the dialog box,
+        # otherwise it may be garbage-collected in another QThread
+        # (e.g. the editor's analysis thread in Spyder), thus leading to
+        # a segmentation fault on UNIX or an application crash on Windows
+        self.setAttribute(Qt.WA_DeleteOnClose)
+        
         assert isinstance(pathlist, list)
         self.pathlist = pathlist
         if ro_pathlist is None:
@@ -208,11 +216,11 @@ class PathManager(QDialog):
     
     def add_path(self):
         self.emit(SIGNAL('redirect_stdio(bool)'), False)
-        directory = QFileDialog.getExistingDirectory(self,
-                                 _("Select directory"), self.last_path)
+        directory = getexistingdirectory(self, _("Select directory"),
+                                         self.last_path)
         self.emit(SIGNAL('redirect_stdio(bool)'), True)
-        if not directory.isEmpty():
-            directory = osp.abspath(unicode(directory))
+        if directory:
+            directory = osp.abspath(directory)
             self.last_path = directory
             if directory in self.pathlist:
                 answer = QMessageBox.question(self, _("Add path"),
