@@ -14,19 +14,20 @@
 import sys, os, time, os.path as osp, re
 
 from spyderlib.qt.QtGui import (QMenu, QApplication, QCursor, QToolTip,
-                                QKeySequence, QFileDialog, QMessageBox,
-                                QMouseEvent, QTextCursor, QTextCharFormat,
-                                QShortcut)
-from spyderlib.qt.QtCore import (Qt, QString, QCoreApplication, SIGNAL,
-                                 pyqtProperty)
+                                QKeySequence, QMessageBox, QMouseEvent,
+                                QTextCursor, QTextCharFormat, QShortcut)
+from spyderlib.qt.QtCore import Qt, QCoreApplication, SIGNAL, Property
+from spyderlib.qt.compat import getsavefilename
 
 # For debugging purpose:
 STDOUT = sys.stdout
 STDERR = sys.stderr
 
 # Local import
-from spyderlib.config import CONF, get_icon, get_conf_path, get_font, _
-from spyderlib.utils import encoding, get_error_match
+from spyderlib.baseconfig import get_conf_path, _
+from spyderlib.config import CONF, get_icon, get_font
+from spyderlib.utils import encoding
+from spyderlib.utils.misc import get_error_match
 from spyderlib.utils.dochelpers import getobj
 from spyderlib.utils.qthelpers import (keybinding, create_action, add_actions,
                                        restore_keyevent)
@@ -197,7 +198,7 @@ class ShellBaseWidget(ConsoleBaseWidget):
             input_buffer = input_buffer.replace(os.linesep, '\n')
         return input_buffer
 
-    input_buffer = pyqtProperty("QString", _get_input_buffer, _set_input_buffer)
+    input_buffer = Property("QString", _get_input_buffer, _set_input_buffer)
         
         
     #------ Prompt
@@ -246,11 +247,11 @@ class ShellBaseWidget(ConsoleBaseWidget):
         """Save current history log (all text in console)"""
         title = _("Save history log")
         self.emit(SIGNAL('redirect_stdio(bool)'), False)
-        filename = QFileDialog.getSaveFileName(self, title,
-                            self.historylog_filename, "History logs (*.log)")
+        filename, _selfilter = getsavefilename(self, title,
+                    self.historylog_filename, "%s (*.log)" % _("History logs"))
         self.emit(SIGNAL('redirect_stdio(bool)'), True)
         if filename:
-            filename = osp.normpath(unicode(filename))
+            filename = osp.normpath(filename)
             try:
                 encoding.write(unicode(self.get_text_with_eol()), filename)
                 self.historylog_filename = filename
@@ -260,7 +261,7 @@ class ShellBaseWidget(ConsoleBaseWidget):
                                      _("<b>Unable to save file '%s'</b>"
                                        "<br><br>Error message:<br>%s"
                                        ) % (osp.basename(filename),
-                                            str(error)))
+                                            unicode(error)))
         
         
     #------ Basic keypress event handler
@@ -334,7 +335,6 @@ class ShellBaseWidget(ConsoleBaseWidget):
             # add and run selection
             else:
                 self.insert_text(self.get_selected_text(), at_end=True)
-            event.accept()
             
         elif key == Qt.Key_Insert and not shift and not ctrl:
             self.setOverwriteMode(not self.overwriteMode())
@@ -345,22 +345,17 @@ class ShellBaseWidget(ConsoleBaseWidget):
                 self.remove_selected_text()
             elif self.is_cursor_on_last_line():
                 self.stdkey_clear()
-            event.accept()
             
         elif key == Qt.Key_Backspace:
             self._key_backspace(cursor_position)
-            event.accept()
             
         elif key == Qt.Key_Tab:
             self._key_tab()
-            event.accept()
             
         elif key == Qt.Key_Space and ctrl:
             self._key_ctrl_space()
-            event.accept()
 
         elif key == Qt.Key_Left:
-            event.accept()
             if self.current_prompt_pos == cursor_position:
                 # Avoid moving cursor on prompt
                 return
@@ -369,7 +364,6 @@ class ShellBaseWidget(ConsoleBaseWidget):
             method('word' if ctrl else 'character', direction='left')
                 
         elif key == Qt.Key_Right:
-            event.accept()
             if self.is_cursor_at_end():
                 return
             method = self.extend_selection_to_next if shift \
@@ -378,11 +372,9 @@ class ShellBaseWidget(ConsoleBaseWidget):
 
         elif (key == Qt.Key_Home) or ((key == Qt.Key_Up) and ctrl):
             self._key_home(shift)
-            event.accept()
 
         elif (key == Qt.Key_End) or ((key == Qt.Key_Down) and ctrl):
             self._key_end(shift)
-            event.accept()
 
         elif key == Qt.Key_Up:
             if not self.is_cursor_on_last_line():
@@ -393,7 +385,6 @@ class ShellBaseWidget(ConsoleBaseWidget):
                 self.stdkey_up(shift)
             else:
                 self.browse_history(backward=True)
-            event.accept()
                 
         elif key == Qt.Key_Down:
             if not self.is_cursor_on_last_line():
@@ -404,7 +395,6 @@ class ShellBaseWidget(ConsoleBaseWidget):
                 self.stdkey_down(shift)
             else:
                 self.browse_history(backward=False)
-            event.accept()
             
         elif key in (Qt.Key_PageUp, Qt.Key_PageDown):
             #XXX: Find a way to do this programmatically instead of calling
@@ -414,49 +404,39 @@ class ShellBaseWidget(ConsoleBaseWidget):
 
         elif key == Qt.Key_Escape:
             self._key_escape()
-            event.accept()
                 
         elif key == Qt.Key_V and ctrl:
             self.paste()
-            event.accept()
             
         elif key == Qt.Key_X and ctrl:
             self.cut()
-            event.accept()
             
         elif key == Qt.Key_Z and ctrl:
             self.undo()
-            event.accept()
             
         elif key == Qt.Key_Y and ctrl:
             self.redo()
-            event.accept()
             
         elif key == Qt.Key_A and ctrl:
             self.selectAll()
-            event.accept()
                 
         elif key == Qt.Key_Question and not self.has_selected_text():
             self._key_question(text)
-            event.accept()
             
         elif key == Qt.Key_ParenLeft and not self.has_selected_text():
             self._key_parenleft(text)
-            event.accept()
             
         elif key == Qt.Key_Period and not self.has_selected_text():
             self._key_period(text)
-            event.accept()
 
-        elif text.length() and not self.isReadOnly():
+        elif len(text) and not self.isReadOnly():
             self.hist_wholeline = False
             self.insert_text(text)
             self._key_other(text)
-            event.accept()
                 
         else:
             # Let the parent widget handle the key press event
-            event.ignore()
+            ConsoleBaseWidget.keyPressEvent(self, event)
             
                 
     #------ Key handlers
@@ -593,7 +573,7 @@ class ShellBaseWidget(ConsoleBaseWidget):
         """Simulate stdout and stderr"""
         if prompt:
             self.flush()
-        if isinstance(text, QString):
+        if not isinstance(text, basestring):
             # This test is useful to discriminate QStrings from decoded str
             text = unicode(text)
         self.__buffer.append(text)
@@ -917,6 +897,9 @@ class PythonShellWidget(ShellBaseWidget):
     def get_completion(self, objtxt):
         """Return completion list associated to object name"""
         pass
+    def get_module_completion(self, objtxt):
+        """Return module completion list associated to object name"""
+        pass
     def get_globals_keys(self):
         """Return shell globals() keys"""
         raise NotImplementedError
@@ -948,9 +931,31 @@ class PythonShellWidget(ShellBaseWidget):
         text = unicode(self.get_current_line_to_cursor())
         last_obj = self.get_last_obj()
         
+        if text.startswith('import '):
+            obj_list = self.get_module_completion(text)
+            words = text.split(' ')
+            if ',' in words[-1]:
+                words = words[-1].split(',')
+            self.show_completion_list(obj_list, completion_text=words[-1],
+                                      automatic=automatic)
+            return
+            
+        elif text.startswith('from '):
+            obj_list = self.get_module_completion(text)
+            if obj_list is None:
+                return
+            words = text.split(' ')
+            if '(' in words[-1]:
+                words = words[:-2] + words[-1].split('(')
+            if ',' in words[-1]:
+                words = words[:-2] + words[-1].split(',')
+            self.show_completion_list(obj_list, completion_text=words[-1],
+                                      automatic=automatic)
+            return
+        
         #-- IPython only -------------------------------------------------------
         # Using IPython code completion feature: __IP.complete
-        if ' ' in text and not text.endswith(' '):
+        elif ' ' in text and not text.endswith(' '):
             try1 = text.split(' ')[-1]
             obj_list = self.get_completion(try1)
             if obj_list:
@@ -978,7 +983,7 @@ class PythonShellWidget(ShellBaseWidget):
             return
         
         # Builtins and globals
-        import re, __builtin__, keyword
+        import __builtin__, keyword
         if not text.endswith('.') and last_obj \
            and re.match(r'[a-zA-Z_0-9]*$', last_obj):
             b_k_g = dir(__builtin__)+self.get_globals_keys()+keyword.kwlist
