@@ -11,14 +11,17 @@
 # pylint: disable=R0911
 # pylint: disable=R0201
 
-import sys, re, string, os
+import sys
+import re
+import sre_constants
+import string
+import os
 
 from spyderlib.qt.QtGui import (QTextCursor, QColor, QFont, QApplication,
                                 QTextEdit, QTextCharFormat, QToolTip,
                                 QTextDocument, QListWidget, QPlainTextEdit,
                                 QPalette, QMainWindow, QTextOption)
-from spyderlib.qt.QtCore import (QPoint, SIGNAL, Qt, QRegExp, QString,
-                                 QEventLoop)
+from spyderlib.qt.QtCore import QPoint, SIGNAL, Qt, QRegExp, QEventLoop
 
 
 # Local imports
@@ -65,25 +68,40 @@ class CompletionWidget(QListWidget):
         # Retrieving current screen height
         desktop = QApplication.desktop()
         srect = desktop.availableGeometry(desktop.screenNumber(self))
-        screen_height = srect.height()
+        screen_right = srect.right()
+        screen_bottom = srect.bottom()
         
         point = self.textedit.cursorRect().bottomRight()
         point.setX(point.x()+self.textedit.get_linenumberarea_width())
         point = self.textedit.mapToGlobal(point)
+
+        # Computing completion widget and its parent right positions
+        comp_right = point.x()+self.width()
+        ancestor = self.parent()
+        if ancestor is None:
+            anc_right = screen_right
+        else:
+            anc_right = min([ancestor.x()+ancestor.width(), screen_right])
+        
+        # Moving completion widget to the left
+        # if there is not enough space to the right
+        if comp_right > anc_right:
+            point.setX(point.x()-self.width())
         
         # Computing completion widget and its parent bottom positions
         comp_bottom = point.y()+self.height()
         ancestor = self.parent()
         if ancestor is None:
-            anc_bottom = screen_height
+            anc_bottom = screen_bottom
         else:
-            anc_bottom = min([ancestor.y()+ancestor.height(), screen_height])
+            anc_bottom = min([ancestor.y()+ancestor.height(), screen_bottom])
         
         # Moving completion widget above if there is not enough space below
+        x_position = point.x()
         if comp_bottom > anc_bottom:
             point = self.textedit.cursorRect().topRight()
-            point.setX(point.x()+self.textedit.get_linenumberarea_width())
             point = self.textedit.mapToGlobal(point)
+            point.setX(x_position)
             point.setY(point.y()-self.height())
             
         if ancestor is not None:
@@ -160,6 +178,7 @@ class TextEditBaseWidget(QPlainTextEdit):
     
     def __init__(self, parent=None):
         QPlainTextEdit.__init__(self, parent)
+        self.setAttribute(Qt.WA_DeleteOnClose)
         
         self.extra_selections_dict = {}
         
@@ -169,6 +188,8 @@ class TextEditBaseWidget(QPlainTextEdit):
 
         self.eol_chars = None
         
+        self.indent_chars = " "*4
+        
         # Code completion / calltips
         if parent is not None:
             mainwin = parent
@@ -201,6 +222,9 @@ class TextEditBaseWidget(QPlainTextEdit):
     def setup_completion(self, size=None, font=None):
         self.completion_widget.setup_appearance(size, font)
         
+    def set_indent_chars(self, indent_chars):
+        self.indent_chars = indent_chars
+        
     def set_palette(self, background, foreground):
         """
         Set text editor palette colors:
@@ -291,8 +315,6 @@ class TextEditBaseWidget(QPlainTextEdit):
             if position > self.get_position('eof'):
                 return
             selection = QTextEdit.ExtraSelection()
-#            selection.format.setProperty(QTextFormat.OutlinePen,
-#                                         QVariant(QPen(color)))
             selection.format.setBackground(color)
             selection.cursor = self.textCursor()
             selection.cursor.clearSelection()
@@ -536,7 +558,7 @@ class TextEditBaseWidget(QPlainTextEdit):
         """
         Set widget end-of-line (EOL) characters from text (analyzes text)
         """
-        if isinstance(text, QString):
+        if not isinstance(text, basestring): # testing for QString (PyQt API#1)
             text = unicode(text)
         eol_chars = sourcecode.get_eol_chars(text)
         if eol_chars is not None and self.eol_chars is not None:
@@ -566,7 +588,7 @@ class TextEditBaseWidget(QPlainTextEdit):
     #------Text selection
     def has_selected_text(self):
         """Returns True if some text is selected"""
-        return not self.textCursor().selectedText().isEmpty()
+        return bool(unicode(self.textCursor().selectedText()))
 
     def get_selected_text(self):
         """
@@ -589,11 +611,17 @@ class TextEditBaseWidget(QPlainTextEdit):
         """Delete selected text"""
         self.textCursor().removeSelectedText()
         
-    def replace(self, text):
-        """Replace selected text by *text*"""
+    def replace(self, text, pattern=None):
+        """Replace selected text by *text*
+        If *pattern* is not None, replacing selected text using regular
+        expression text substitution"""
         cursor = self.textCursor()
         cursor.beginEditBlock()
+        if pattern is not None:
+            seltxt = unicode(cursor.selectedText())
         cursor.removeSelectedText()
+        if pattern is not None:
+            text = re.sub(unicode(pattern), unicode(text), unicode(seltxt))
         cursor.insertText(text)
         cursor.endEditBlock()
         
@@ -609,7 +637,13 @@ class TextEditBaseWidget(QPlainTextEdit):
 
     def get_text_line(self, line_nb):
         """Return text line at line number *line_nb*"""
-        return unicode(self.toPlainText()).splitlines()[line_nb-1]
+        # Taking into account the case when a file ends in an empty line,
+        # since splitlines doesn't return that line as the last element
+        # TODO: Make this function more efficient
+        try:
+            return unicode(self.toPlainText()).splitlines()[line_nb]
+        except IndexError:
+            return self.get_line_separator()
     
     def get_text(self, position_from, position_to):
         """
@@ -617,13 +651,13 @@ class TextEditBaseWidget(QPlainTextEdit):
         Positions may be positions or 'sol', 'eol', 'sof', 'eof' or 'cursor'
         """
         cursor = self.__select_text(position_from, position_to)
-        text = cursor.selectedText()
-        if not text.isEmpty():
-            while text.endsWith("\n"):
-                text.chop(1)
-            while text.endsWith(u"\u2029"):
-                text.chop(1)
-        return unicode(text)
+        text = unicode(cursor.selectedText())
+        if text:
+            while text.endswith("\n"):
+                text = text[:-1]
+            while text.endswith(u"\u2029"):
+                text = text[:-1]
+        return text
     
     def get_character(self, position):
         """Return character at *position*"""
@@ -640,7 +674,8 @@ class TextEditBaseWidget(QPlainTextEdit):
     
     def insert_text(self, text):
         """Insert text at cursor position"""
-        self.textCursor().insertText(text)
+        if not self.isReadOnly():
+            self.textCursor().insertText(text)
     
     def replace_text(self, position_from, position_to, text):
         cursor = self.__select_text(position_from, position_to)
@@ -650,9 +685,39 @@ class TextEditBaseWidget(QPlainTextEdit):
     def remove_text(self, position_from, position_to):
         cursor = self.__select_text(position_from, position_to)
         cursor.removeSelectedText()
+        
+    def find_multiline_pattern(self, regexp, cursor, findflag):
+        """Reimplement QTextDocument's find method
+        
+        Add support for *multiline* regular expressions"""
+        pattern = unicode(regexp.pattern())
+        text = unicode(self.toPlainText())
+        try:
+            regobj = re.compile(pattern)
+        except sre_constants.error:
+            return
+        if findflag & QTextDocument.FindBackward:
+            # Find backward
+            offset = min([cursor.selectionEnd(), cursor.selectionStart()])
+            text = text[:offset]
+            matches = [_m for _m in regobj.finditer(text, 0, offset)]
+            if matches:
+                match = matches[-1]
+            else:
+                return
+        else:
+            # Find forward
+            offset = max([cursor.selectionEnd(), cursor.selectionStart()])
+            match = regobj.search(text, offset)
+        if match:
+            pos1, pos2 = match.span()
+            fcursor = self.textCursor()
+            fcursor.setPosition(pos1)
+            fcursor.setPosition(pos2, QTextCursor.KeepAnchor)
+            return fcursor
 
-    def find_text(self, text, changed=True,
-                  forward=True, case=False, words=False, regexp=False):
+    def find_text(self, text, changed=True, forward=True, case=False,
+                  words=False, regexp=False):
         """Find text"""
         cursor = self.textCursor()
         findflag = QTextDocument.FindFlag()
@@ -662,22 +727,30 @@ class TextEditBaseWidget(QPlainTextEdit):
         if forward:
             moves += [QTextCursor.NextWord, QTextCursor.Start]
             if changed:
-                if cursor.selectedText().isEmpty():
-                    cursor.movePosition(QTextCursor.PreviousWord)
-                else:
+                if unicode(cursor.selectedText()):
                     new_position = min([cursor.selectionStart(),
                                         cursor.selectionEnd()])
                     cursor.setPosition(new_position)
+                else:
+                    cursor.movePosition(QTextCursor.PreviousWord)
         else:
             moves += [QTextCursor.End]
         if not regexp:
             text = re.escape(unicode(text))
-        regexp = QRegExp(r"\b%s\b" % text if words else text,
-                         Qt.CaseSensitive if case else Qt.CaseInsensitive)
+        pattern = QRegExp(r"\b%s\b" % text if words else text,
+                          Qt.CaseSensitive if case else Qt.CaseInsensitive,
+                          QRegExp.RegExp2)
         for move in moves:
             cursor.movePosition(move)
-            found_cursor = self.document().find(regexp, cursor, findflag)
-            if not found_cursor.isNull():
+            if regexp and '\\n' in text:
+                # Multiline regular expression
+                found_cursor = self.find_multiline_pattern(pattern, cursor,
+                                                           findflag)
+            else:
+                # Single line find: using the QTextDocument's find function,
+                # probably much more efficient than ours
+                found_cursor = self.document().find(pattern, cursor, findflag)
+            if found_cursor is not None and not found_cursor.isNull():
                 self.setTextCursor(found_cursor)
                 return True
         return False
@@ -714,7 +787,7 @@ class TextEditBaseWidget(QPlainTextEdit):
         cursor.select(QTextCursor.WordUnderCursor)
         return unicode(cursor.selectedText())
     
-    def get_indentation(self, block_nb):
+    def get_block_indentation(self, block_nb):
         """Return line indentation (character number)"""
         text = unicode(self.document().findBlockByNumber(block_nb).text())
         return len(text)-len(text.lstrip())
@@ -727,6 +800,51 @@ class TextEditBaseWidget(QPlainTextEdit):
         block_end = self.document().findBlock(end)
         return sorted([block_start.blockNumber(), block_end.blockNumber()])
         
+    def get_executable_text(self):
+        """Return selected text or current line as an processed text,
+        to be executable in a Python/IPython interpreter"""
+        no_selection = not self.has_selected_text()
+        if no_selection:
+            self.select_current_block()
+        
+        ls = self.get_line_separator()
+        
+        _indent = lambda line: len(line)-len(line.lstrip())
+        
+        line_from, line_to = self.get_selection_bounds()
+        text = self.get_selected_text()
+
+        lines = text.split(ls)
+        if len(lines) > 1:
+            # Multiline selection -> eventually fixing indentation
+            original_indent = _indent(self.get_text_line(line_from))
+            text = (" "*(original_indent-_indent(lines[0])))+text
+        
+        # If there is a common indent to all lines, remove it
+        min_indent = 999
+        for line in text.split(ls):
+            if line.strip():
+                min_indent = min(_indent(line), min_indent)
+        if min_indent:
+            text = ls.join([line[min_indent:] for line in text.split(ls)])
+        
+        # Add an EOL character if a block stars with various Python
+        # reserved words, so that it gets evaluated automatically
+        # by the console
+        first_line = lines[0].lstrip()
+        last_line = self.get_text_line(line_to).strip()
+        words = ['def', 'for', 'if', 'while', 'try', 'with', 'class']
+        if any([first_line.startswith(w) for w in words]):
+            text += ls
+            if last_line != '':
+                text += ls
+
+        if no_selection:
+            self.setFocus()
+            self.clear_selection()
+        
+        return text
+    
     def get_line_count(self):
         """Return document total line number"""
         return self.blockCount()
@@ -737,13 +855,13 @@ class TextEditBaseWidget(QPlainTextEdit):
         cursor.beginEditBlock()
         orig_sel = start_pos, end_pos = (cursor.selectionStart(),
                                          cursor.selectionEnd())
-        if not cursor.selectedText().isEmpty():
+        if unicode(cursor.selectedText()):
             cursor.setPosition(end_pos)
             # Check if end_pos is at the start of a block: if so, starting
             # changes from the previous block
             cursor.movePosition(QTextCursor.StartOfBlock,
                                 QTextCursor.KeepAnchor)
-            if cursor.selectedText().isEmpty():
+            if not unicode(cursor.selectedText()):
                 cursor.movePosition(QTextCursor.PreviousBlock)
                 end_pos = cursor.position()
             
@@ -786,6 +904,61 @@ class TextEditBaseWidget(QPlainTextEdit):
         """
         self.__duplicate_line_or_selection(after_current_line=False)
         
+    def __move_line_or_selection(self, after_current_line=True):
+        """Move current line or selected text"""
+        cursor = self.textCursor()
+        cursor.beginEditBlock()
+        orig_sel = start_pos, end_pos = (cursor.selectionStart(),
+                                         cursor.selectionEnd())
+        if unicode(cursor.selectedText()):
+            # Check if start_pos is at the start of a block
+            cursor.setPosition(start_pos)
+            cursor.movePosition(QTextCursor.StartOfBlock)
+            start_pos = cursor.position()
+
+            cursor.setPosition(end_pos)
+            # Check if end_pos is at the start of a block: if so, starting
+            # changes from the previous block
+            cursor.movePosition(QTextCursor.StartOfBlock,
+                                QTextCursor.KeepAnchor)
+            if unicode(cursor.selectedText()):
+                cursor.movePosition(QTextCursor.NextBlock)
+                end_pos = cursor.position()
+        else:
+            cursor.movePosition(QTextCursor.StartOfBlock)
+            start_pos = cursor.position()
+            cursor.movePosition(QTextCursor.NextBlock)
+            end_pos = cursor.position()
+        cursor.setPosition(start_pos)
+        cursor.setPosition(end_pos, QTextCursor.KeepAnchor)
+        
+        sel_text = unicode(cursor.selectedText())
+        cursor.removeSelectedText()
+        
+        if after_current_line:
+            text = unicode(cursor.block().text())
+            orig_sel = (orig_sel[0]+len(text)+1, orig_sel[1]+len(text)+1)
+            cursor.movePosition(QTextCursor.NextBlock)
+        else:
+            cursor.movePosition(QTextCursor.PreviousBlock)
+            text = unicode(cursor.block().text())
+            orig_sel = (orig_sel[0]-len(text)-1, orig_sel[1]-len(text)-1)
+        cursor.insertText(sel_text)
+
+        cursor.endEditBlock()
+
+        cursor.setPosition(orig_sel[0])
+        cursor.setPosition(orig_sel[1], QTextCursor.KeepAnchor)
+        self.setTextCursor(cursor)
+    
+    def move_line_up(self):
+        """Move up current line or selected text"""
+        self.__move_line_or_selection(after_current_line=False)
+        
+    def move_line_down(self):
+        """Move down current line or selected text"""
+        self.__move_line_or_selection(after_current_line=True)
+        
     def extend_selection_to_complete_lines(self):
         """Extend current selection to complete lines"""
         cursor = self.textCursor()
@@ -839,8 +1012,7 @@ class TextEditBaseWidget(QPlainTextEdit):
         format2 = '<hr><div style=\'font-family: "%s"; font-size: %spt; font-weight: %s\'>' % (family, size, weight)
         if isinstance(text, list):
             text = "\n    ".join(text)
-        else:
-            text = text.replace('\n', '<br>')
+        text = text.replace('\n', '<br>')
         if len(text) > self.calltip_size:
             text = text[:self.calltip_size] + " ..."
         tiptext = format1 + ('<b>%s</b></div>' % title) \
@@ -883,8 +1055,12 @@ class TextEditBaseWidget(QPlainTextEdit):
         if len(completions) == 0 or completions == [completion_text]:
             return
         self.completion_text = completion_text
-        self.show_completion_widget(sorted(completions, key=string.lower),
-                                    automatic=automatic)
+        # Sorting completion list (entries starting with underscore are 
+        # put at the end of the list):
+        underscore = set([comp for comp in completions if comp.startswith('_')])
+        completions = sorted(set(completions)-underscore, key=string.lower)+\
+                      sorted(underscore, key=string.lower)
+        self.show_completion_widget(completions, automatic=automatic)
         
     def select_completion_list(self):
         """Completion list is active, Enter was just pressed"""
@@ -926,7 +1102,7 @@ class TextEditBaseWidget(QPlainTextEdit):
         self.moveCursor(QTextCursor.Down, self.__get_move_mode(shift))
 
     def stdkey_tab(self):
-        self.insert_text(" "*4)
+        self.insert_text(self.indent_chars)
 
     def stdkey_home(self, shift, prompt_pos=None):
         """Smart HOME feature: cursor is first moved at 
