@@ -13,20 +13,18 @@
 
 from __future__ import with_statement
 
-try:
-    # PyQt4 4.3.3 on Windows (static DLLs) with py2exe installed:
-    # -> pythoncom must be imported first, otherwise py2exe's boot_com_servers
-    #    will raise an exception ("ImportError: DLL load failed [...]") when
-    #    calling any of the QFileDialog static methods (getOpenFileName, ...)
-    import pythoncom #@UnusedImport
-except ImportError:
-    pass
-
 from spyderlib.qt.QtGui import (QHBoxLayout, QWidget, QTreeWidgetItem,
-                                QMessageBox, QVBoxLayout, QLabel, QFileDialog)
-from spyderlib.qt.QtCore import SIGNAL, QProcess, QByteArray, QString
+                                QMessageBox, QVBoxLayout, QLabel)
+from spyderlib.qt.QtCore import SIGNAL, QProcess, QByteArray, QTextCodec
+locale_codec = QTextCodec.codecForLocale()
+from spyderlib.qt.compat import getopenfilename
 
-import sys, os, time, cPickle, os.path as osp, re
+import sys
+import os
+import os.path as osp
+import time
+import cPickle
+import re
 
 # For debugging purpose:
 STDOUT = sys.stdout
@@ -34,19 +32,19 @@ STDOUT = sys.stdout
 # Local imports
 from spyderlib.utils import programs
 from spyderlib.utils.qthelpers import create_toolbutton
-from spyderlib.config import get_icon, get_conf_path
+from spyderlib.baseconfig import get_conf_path, get_translation
+from spyderlib.config import get_icon
 from spyderlib.widgets.onecolumntree import OneColumnTree
 from spyderlib.widgets.texteditor import TextEditor
 from spyderlib.widgets.comboboxes import (PythonModulesComboBox,
                                           is_module_or_package)
-from spyderlib.utils.translations import get_translation
 _ = get_translation("p_pylint", dirname="spyderplugins")
 
-PYLINT_PATH = programs.get_nt_program_name('pylint')
 
-def is_pylint_installed():
-    return programs.is_program_installed(PYLINT_PATH)
+PYLINT_PATH = programs.find_program('pylint')
 
+
+#TODO: display results on 3 columns instead of 1: msg_id, lineno, message
 class ResultsTree(OneColumnTree):
     def __init__(self, parent):
         OneColumnTree.__init__(self, parent)
@@ -93,7 +91,7 @@ class ResultsTree(OneColumnTree):
             if not messages:
                 title_item.setDisabled(True)
             modules = {}
-            for module, lineno, message in messages:
+            for module, lineno, message, msg_id in messages:
                 basename = osp.splitext(osp.basename(self.filename))[0]
                 if not module.startswith(basename):
                     # Pylint bug
@@ -120,9 +118,11 @@ class ResultsTree(OneColumnTree):
                         parent = item
                 else:
                     parent = title_item
-                msg_item = QTreeWidgetItem(parent,
-                                           ["%d : %s" % (lineno, message)],
-                                           QTreeWidgetItem.Type)
+                if len(msg_id) > 1:
+                    text = "[%s] %d : %s" % (msg_id, lineno, message)
+                else:
+                    text = "%d : %s" % (lineno, message)
+                msg_item = QTreeWidgetItem(parent, [text], QTreeWidgetItem.Type)
                 msg_item.setIcon(0, get_icon('arrow.png'))
                 self.data[msg_item] = (modname, lineno)
 
@@ -132,7 +132,7 @@ class PylintWidget(QWidget):
     Pylint widget
     """
     DATAPATH = get_conf_path('.pylint.results')
-    VERSION = '1.0.2'
+    VERSION = '1.1.0'
     
     def __init__(self, parent, max_entries=100):
         QWidget.__init__(self, parent)
@@ -206,7 +206,7 @@ class PylintWidget(QWidget):
         self.process = None
         self.set_running_state(False)
         
-        if not is_pylint_installed():
+        if PYLINT_PATH is None:
             for widget in (self.treewidget, self.filecombo,
                            self.start_button, self.stop_button):
                 widget.setDisabled(True)
@@ -225,7 +225,7 @@ class PylintWidget(QWidget):
             self.show_data()
         
     def analyze(self, filename):
-        if not is_pylint_installed():
+        if PYLINT_PATH is None:
             return
         filename = unicode(filename) # filename is a QString instance
         self.kill_if_running()
@@ -241,9 +241,8 @@ class PylintWidget(QWidget):
             
     def select_file(self):
         self.emit(SIGNAL('redirect_stdio(bool)'), False)
-        filename = QFileDialog.getOpenFileName(self,
-                                      _("Select Python script"), os.getcwdu(),
-                                      _("Python scripts")+" (*.py ; *.pyw)")
+        filename, _selfilter = getopenfilename(self, _("Select Python script"),
+                           os.getcwdu(), _("Python scripts")+" (*.py ; *.pyw)")
         self.emit(SIGNAL('redirect_stdio(bool)'), False)
         if filename:
             self.analyze(filename)
@@ -299,7 +298,7 @@ class PylintWidget(QWidget):
         
         self.output = ''
         self.error_output = ''
-        p_args = [osp.basename(filename)]
+        p_args = ['-i', 'yes', osp.basename(filename)]
         self.process.start(PYLINT_PATH, p_args)
         
         running = self.process.waitForStarted()
@@ -323,7 +322,7 @@ class PylintWidget(QWidget):
                 bytes += self.process.readAllStandardError()
             else:
                 bytes += self.process.readAllStandardOutput()
-        text = unicode( QString.fromLocal8Bit(bytes.data()) )
+        text = unicode( locale_codec.toUnicode(bytes.data()) )
         if error:
             self.error_output += text
         else:
@@ -350,16 +349,17 @@ class PylintWidget(QWidget):
             i1 = line.find(':')
             if i1 == -1:
                 continue
+            msg_id = line[:i1]
             i2 = line.find(':', i1+1)
             if i2 == -1:
                 continue
             line_nb = line[i1+1:i2].strip()
             if not line_nb:
                 continue
-            line_nb = int(line_nb)
+            line_nb = int(line_nb.split(',')[0])
             message = line[i2+1:]
-            item = (module, line_nb, message)
-            results[line[0]+':'].append(item)                
+            item = (module, line_nb, message, msg_id)
+            results[line[0]+':'].append(item)
             
         # Rate
         rate = None
